package lib.applets;

/**
 * @class lib.applets
 * 
 * This class is used to store the state of the "view" 
 * in the applet GUI for managing areas
 * 
 * @author Tihomir Lazarov
 * @date   Jun 10, 2005
 */	
public class AreaView
{
	// The list of available modes in the view
	public static final int AREA_VIEW_MODE_NONE		= 0;
	public static final int AREA_VIEW_MODE_NEW		= 1;
	public static final int AREA_VIEW_MODE_UPDATE	= 2;
	public static final int AREA_VIEW_MODE_REMOVE	= 3;
	
	int m_iMode;			// The current mode
	long m_lAreaID;		// The currently edited area ID
	
	/**
	 * Constructor. Resets the current view
	 */
	public AreaView( ) { reset( ); }

	/**
	 * Changes the area view mode and sets the ID of the
	 * currently edited area to zero
	 * 
	 * @param iMode - the mode to set
	 */	
	public void setMode( int iMode )
	{
		m_iMode 		= iMode;
		m_lAreaID	= 0;
	}
	
	/**
	 * Sets the current area ID to another value
	 * 
	 * @param lAreaID - the new area ID value
	 */
	public void setAreaID( long lAreaID )
	{
		m_lAreaID = lAreaID;
	}
	
	/**
	 * Resets the current area view mode
	 */
	public void reset( )
	{
		setMode( AREA_VIEW_MODE_NONE );
	}
	
	/**
	 * Returns the current area view mode
	 * 
	 * @return the current mode
	 */
	public int getMode( )
	{
		return m_iMode;
	}
	
	public long getAreaID( )
	{
		return m_lAreaID;
	}
}	
