package lib.applets;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.awt.Graphics;
import java.util.Enumeration;
import java.util.Hashtable;
import java.net.MalformedURLException;
import java.net.URL;
import javax.swing.JComboBox;

import lib.cfg.Constants;
import lib.comm.AreaInfo;
import lib.comm.IconInfo;
import lib.comm.GroupInfo;
import lib.comm.MapDefinition;
import lib.comm.MapRequest;
import lib.comm.MapRequestInfo;
import lib.exceptions.MapRequestException;

/**
 * @class lib.applets
 * 
 * Map navigation and management applet user interface
 * 
 * @author Tihomir Lazarov * @date   Jun 4, 2005
 */
public class MapApplet extends MapGUI 
							  implements ActionListener, 
							             MouseListener,
							             MouseMotionListener {
		
	/** -- CONSTANTS ------------------------------------------------------- */
	static final int MAP_ACTION_INIT				= 1;
	static final int MAP_ACTION_ZOOM_IN			= 2;
	static final int MAP_ACTION_ZOOM_OUT		= 3;
	static final int MAP_ACTION_MOVE_LEFT		= 4;
	static final int MAP_ACTION_MOVE_UP			= 5;
	static final int MAP_ACTION_MOVE_RIGHT		= 6;
	static final int MAP_ACTION_MOVE_DOWN		= 7;
	static final int MAP_ACTION_ZOOM_IN_MOVE	= 8;
	static final int MAP_ACTION_ZOOM_OUT_MOVE	= 9;
	static final int MAP_ACTION_ADD_AREA		= 10;
	static final int MAP_ACTION_UPDATE_AREA	= 11;
	static final int MAP_ACTION_GET_AREA		= 12;
	static final int MAP_ACTION_REMOVE_AREA	= 13;
	static final int MAP_ACTION_SAVE_MAP		= 14;
	static final int MAP_ACTION_CLICK			= 15;
	static final int MAP_ACTION_SEARCH_MAP		= 16;
	static final int MAP_ACTION_LOGIN			= 17;
	static final int MAP_ACTION_LOGOUT			= 18;
	static final int MAP_ACTION_ZOOM_TO			= 19;
	
	static final Cursor HAND_CURSOR 				= new Cursor( Cursor.HAND_CURSOR );
	static final Cursor DEFAULT_CURSOR 			= new Cursor( Cursor.DEFAULT_CURSOR );
	
	// Current map snapshot
	BufferedImage m_MapImage 						= null;
	
	// The map information that's received upon loading of a map
	MapRequestInfo m_InitMapInfo					= null;
	
	// The last map information received (usually when navigating over the
	// map this information is updated
	MapRequestInfo m_LastMapInfo					= null;
	
	// The object that manages the requests to the servlets
	MapRequest m_MapRequest							= null;
	
	// This flag is used as a mutex to prevent multiple map actions 
	// executed at one time
	boolean m_bMapAction								= false;
	
	// Contains the state of the area view
	AreaView m_AreaView								= new AreaView( );
	
	// Contains the state of the search view
	SearchView m_SearchView							= new SearchView( );
	 	
	// Current view
	String m_szCurrentView 							= TOOLBAR_VIEW_SEARCH;
	
	TooltipInfo m_TooltipInfo						= new TooltipInfo( );

	public void init( )
	{
		super.init( );
		//////////////////////////////////////////////
		// ADD ACTION LISTENERS
		//////////////////////////////////////////////
		m_ZoomInButton.addActionListener( this );
		m_ZoomOutButton.addActionListener( this );
		m_SmoothZoomCanvas.addMouseListener( this );
		m_MoveLeftButton.addActionListener( this );
		m_MoveUpButton.addActionListener( this );
		m_MoveRightButton.addActionListener( this );
		m_MoveDownButton.addActionListener( this );
		m_Canvas.addMouseListener( this );
		m_Canvas.addMouseMotionListener( this );
		
		m_MapChoice.addActionListener( this );
		m_SaveMapButton.addActionListener( this );
		m_LogoutButton.addActionListener( this );
		
		m_SearchButton.addActionListener( this );
		m_SearchCancelButton.addActionListener( this );
		m_ResetButton.addActionListener( this );
		
		m_ViewSearchButton.addActionListener( this );
		m_ViewAreasButton.addActionListener( this );
		m_NewAreaButton.addActionListener( this );
		m_UpdateAreaButton.addActionListener( this );
		m_RemoveAreaButton.addActionListener( this );
		m_OKAreaButton.addActionListener( this );
		m_CancelAreaButton.addActionListener( this );
		
		m_OKAuthButton.addActionListener( this );
		m_CancelAuthButton.addActionListener( this );
	}
	
	/**
	 * Overrides the "start" of the applet
	 */	
	public void start( )
	{
		super.start( );
		
		if ( m_MapImage == null ) {
			// Load the applet stuff in case we're invoking start for the first time.
			// The loading is done here, because if I do it on 'init' the screen will be 
			// blank until I get the information from the server
			load( );
		}		
		
		repaint( );
	}
	
	/**
	 * Button action handler
	 * 
	 * @param event - the ActionEvent
	 */
	public void actionPerformed( ActionEvent event )
	{
		if ( m_InitMapInfo == null ) return;
		
		// ZOOM IN
		if ( event.getSource( ).equals( m_ZoomInButton ) ) {
			mapAction( MAP_ACTION_ZOOM_IN, null );
		// ZOOM OUT
		} else if ( event.getSource( ).equals( m_ZoomOutButton ) ) {
			mapAction( MAP_ACTION_ZOOM_OUT, null );
		// MOVE LEFT
		} else if ( event.getSource( ).equals( m_MoveLeftButton ) ) {
			mapAction( MAP_ACTION_MOVE_LEFT, null );
		// MOVE UP
		} else if ( event.getSource( ).equals( m_MoveUpButton ) ) {
			mapAction( MAP_ACTION_MOVE_UP, null );
		// MOVE DOWN
		} else if ( event.getSource( ).equals( m_MoveDownButton ) ) {
			mapAction( MAP_ACTION_MOVE_DOWN, null );
		// MOVE RIGHT
		} else if ( event.getSource( ).equals( m_MoveRightButton ) ) {
			mapAction( MAP_ACTION_MOVE_RIGHT, null );
		// CHOOSE A MAP
		} else if ( event.getSource( ).equals( m_MapChoice ) ) {
			MapDefinition selectedItem = ( MapDefinition )m_MapChoice.getSelectedItem( );
			Hashtable params = new Hashtable( );
			params.put( Constants.PARAM_MAP_ID, "" + selectedItem.id );
			setView( TOOLBAR_VIEW_SEARCH );
			mapAction( MAP_ACTION_INIT, params );
		// SEARCH VIEW
		} else if ( event.getSource( ).equals( m_ViewSearchButton ) ) {
			setView( TOOLBAR_VIEW_SEARCH );
		// AREA VIEW
		} else if ( event.getSource( ).equals( m_ViewAreasButton ) ) {
			if ( m_LastMapInfo.isLogged( ) ) {
				setView( TOOLBAR_VIEW_AREAS );
			} else {
				setView( TOOLBAR_VIEW_LOGIN );
			}
		// NEW AREA MODE
		} else if ( event.getSource( ).equals( m_NewAreaButton ) ) {
			m_AreaView.setMode( AreaView.AREA_VIEW_MODE_NEW );
			resetAreaFields( );
			setAreaGUIState( );
		// UPDATE AREA MODE
		} else if ( event.getSource( ).equals( m_UpdateAreaButton ) ) {
			m_AreaView.setMode( AreaView.AREA_VIEW_MODE_UPDATE );
			resetAreaFields( );
			setAreaGUIState( );
		// REMOVE AREA MODE
		} else if ( event.getSource( ).equals( m_RemoveAreaButton ) ) {
			m_AreaView.setMode( AreaView.AREA_VIEW_MODE_REMOVE );
			resetAreaFields( );
			setAreaGUIState( );
		// OK in the area panel
		} else if ( event.getSource( ).equals( m_OKAreaButton ) ) {
			if ( clickOKAreaButton( ) ) {
				resetAreaFields( );
				m_AreaView.reset( );
				setAreaGUIState( );
			}
		// CANCEL in the area panel
		} else if ( event.getSource( ).equals( m_CancelAreaButton) ) {
			resetAreaFields( );
			m_AreaView.reset( );
			setAreaGUIState( );
		// SAVE MAP
		} else if ( event.getSource( ).equals( m_SaveMapButton ) ) {
			mapAction( MAP_ACTION_SAVE_MAP, null );
		// LOGOUT
		} else if ( event.getSource( ).equals( m_LogoutButton ) ) {
			mapAction( MAP_ACTION_LOGOUT, null );
		// SEARCH
		} else if ( event.getSource( ).equals( m_SearchButton ) ) {
			mapAction( MAP_ACTION_SEARCH_MAP, null );
		// CANCEL search
		} else if ( event.getSource( ).equals( m_SearchCancelButton ) ) {
			setView( TOOLBAR_VIEW_NONE );
		// RESET search
		} else if ( event.getSource( ).equals( m_ResetButton ) ) {
			resetSearch( );
			mapAction( MAP_ACTION_SEARCH_MAP, null );
		// LOGIN button
		} else if ( event.getSource( ).equals( m_OKAuthButton ) ) {
		// CANCEL login buttno
			mapAction( MAP_ACTION_LOGIN, null );
		} else if ( event.getSource( ).equals( m_CancelAuthButton ) ) {
			setView( TOOLBAR_VIEW_SEARCH );
		}
	}
	
	/**
	 * Handles clicking of a mouse button
	 *
	 * @param event - the MouseEvent
	 */
	public void mouseClicked ( MouseEvent event ) 
	{
		if ( m_InitMapInfo == null ) return;
		
		if ( event.getSource( ).equals( m_Canvas ) ) {
			boolean bDoZoom 	= false;
			int x					= event.getX( );
			int y					= event.getY( );
	
			// If the toolbar view is "area" the behavior is the following:
			// When we're not clicking over an area, the coordinates
			// that we click are filled in the X and Y fields.
			// If an area is clicked, a request is made and the 
			// area information is fetched and filled in the
			// area fields
			if ( m_szCurrentView.equals( TOOLBAR_VIEW_AREAS ) ) {
				boolean bGetAreaInfo = false;
				switch ( m_AreaView.getMode( ) ) {
					case AreaView.AREA_VIEW_MODE_NEW:
						setAreaCoorFields( event.getX( ), event.getY( ) );
						break;
					case AreaView.AREA_VIEW_MODE_UPDATE:
						if ( m_AreaView.getAreaID( ) > 0 ) {
							setAreaCoorFields( event.getX( ), event.getY( ) );
						} else {
							bGetAreaInfo = true;
						}
						break;
					case AreaView.AREA_VIEW_MODE_REMOVE:
						bGetAreaInfo = true;
						break;
					case AreaView.AREA_VIEW_MODE_NONE:
						// Work as in search mode
						bDoZoom = true;
						break;
				}
				
				if ( bGetAreaInfo ) {
					AreaInfo areaInfo = getAreaOnPos( x, y );
					if ( areaInfo != null ) {
						Hashtable params = new Hashtable( );
						params.put( Constants.PARAM_X, "" + x );
						params.put( Constants.PARAM_Y, "" + y );
						mapAction( MAP_ACTION_GET_AREA, params );
					}
				}
			}
			
			if ( m_szCurrentView.equals( TOOLBAR_VIEW_SEARCH ) || 
			     m_szCurrentView.equals( TOOLBAR_VIEW_LOGIN  ) ) {
				bDoZoom = true;
			}
			
			if ( bDoZoom ) {
				AreaInfo areaInfo = getAreaOnPos( x, y );
				
				//	Execute zoo & move action if we're not clicking
				// over an area
				Hashtable params 	= new Hashtable( );
				boolean bJustZoom	= false;
				
				params.put( Constants.PARAM_X, "" + event.getX( ) );
				params.put( Constants.PARAM_Y, "" + event.getY( ) );
				
				// No area found, just zoom...
				if ( areaInfo == null ) {
					bJustZoom = true;
				} else {
					// We're clicking on an area
					// If a CTRL key is pressed, just add the area to the
					// search view. If the SHIFT key is pressed execute the ON CLICK
					// event over the area. In the other cases work like
					// zooming
					if ( event.getModifiersEx( ) == InputEvent.CTRL_DOWN_MASK ) {
						m_SearchView.setArea( areaInfo );
						setShortestPathSearchControls( );
					} else if ( event.getModifiersEx( ) == InputEvent.SHIFT_DOWN_MASK ) {
						mapAction( MAP_ACTION_CLICK, params );
					} else {
						bJustZoom = true;
					}
				}
				
				if ( bJustZoom ) {
					if ( event.getButton() == MouseEvent.BUTTON1 ) {
						mapAction( MAP_ACTION_ZOOM_IN_MOVE, params );
					} else {
						mapAction( MAP_ACTION_ZOOM_OUT_MOVE, params );
					}
				}
			}
		} // If the event comes form the m_Canvas
		
		// Clicking on the zoom area
		if ( event.getSource( ).equals( m_SmoothZoomCanvas ) ) {
			int x = event.getX( );
			int y = event.getY( );
			
			// Calculate the clicked zoom level and zoom to it 
			// only if it's different than the current zoom level
			int iClickedZoomLevel = ( int )( ( ( float ) x / m_SmoothZoomCanvas.getWidth( ) ) * 
			                          m_InitMapInfo.getZoomLevels( ) );
			if ( iClickedZoomLevel != m_LastMapInfo.getZoomLevel( ) &&
			     iClickedZoomLevel < m_InitMapInfo.getZoomLevels( ) ) {
				Hashtable params = new Hashtable( );
				params.put( Constants.PARAM_ZOOM_LEVEL, "" + iClickedZoomLevel );
				mapAction( MAP_ACTION_ZOOM_TO, params );
			}
		}
	}
	
	/**
	 * A handler for mouse movements
	 * 
	 * @param event - the event of moving the mouse
	 */
	public void mouseMoved( MouseEvent event )
	{
		// Check if the mouse is over an area and change the
		// cursor to a hand
		if ( m_LastMapInfo == null ) return;
		int x 				= event.getX( );
		int y 				= event.getY( );
		AreaInfo areaInfo = getAreaOnPos( x, y );
		if ( areaInfo != null ) {
			String szTooltipText = "";
			String szName			= ( String ) areaInfo.info.get( Constants.AREA_ATTRIBUTE_NAME );
			String szDescription	= ( String ) areaInfo.info.get( Constants.AREA_ATTRIBUTE_DESCRIPTION );
			String szURL			= ( String ) areaInfo.info.get( Constants.AREA_ATTRIBUTE_URL );
			m_TooltipInfo.reset( );
			if ( szName 			!= null ) m_TooltipInfo.name 			= szName;
			if ( szDescription 	!= null ) m_TooltipInfo.description = szDescription;
			if ( szURL 				!= null ) m_TooltipInfo.url 			= szURL;
			m_TooltipInfo.x = x;
			m_TooltipInfo.y = y;
			
			m_Canvas.setCursor( HAND_CURSOR );
		} else {
			m_TooltipInfo.reset( );
			m_Canvas.setCursor( DEFAULT_CURSOR );
		}
		repaint( );
	}
	
	public void mouseEntered ( MouseEvent event ) {}
	public void mouseExited  ( MouseEvent event ) {}
	public void mousePressed ( MouseEvent event ) {}
	public void mouseReleased( MouseEvent event ) {}
	public void mouseDragged ( MouseEvent event ) {}
		
		
	/**
	 * Standard "paint" method of the applet
	 * 
	 * @param g - applet Graphics object reference
	 */	
	public void paint( Graphics g )
	{
		if ( m_MapImage == null ) { return; }
				
		update( g );
		
		super.paint( g );
	}
	
	/**
	 * Standard 'update' method for Java Applets
	 * 
	 * @param g - the applet's Graphics object
	 */
	public void update( Graphics g ) 
	{
		if ( m_MapImage == null ) { return; }
		
		//	Draw the current map		
		Graphics canvas = m_Canvas.getGraphics( );

		Image image = createImage( m_MapImage.getWidth( ), m_MapImage.getHeight( ) );
		Graphics2D imageGraphics = ( Graphics2D )image.getGraphics( );
		imageGraphics.drawImage( m_MapImage, 0, 0, this );
		if ( m_TooltipInfo.isSet( ) ) {
			int x = m_TooltipInfo.x + 10;
			int y = m_TooltipInfo.y + 10;
			BufferedImage tooltipImage = m_TooltipInfo.createImage( getGraphics( ).getFontMetrics() );
			int w = tooltipImage.getWidth( );
			int h = tooltipImage.getHeight( );
			if ( x + w > m_Canvas.getWidth( ) )  x = m_Canvas.getWidth( ) - w;
			if ( y + h > m_Canvas.getHeight( ) ) y = m_Canvas.getHeight( ) - h;
			imageGraphics.drawImage( tooltipImage, x, y, this );
		}
		
		Image canvasImage = createImage( m_Canvas.getWidth( ), m_Canvas.getHeight( ) );
		Graphics canvasGraphics = canvasImage.getGraphics( );
		canvasGraphics.clearRect( 0, 0, m_Canvas.getWidth( ), m_Canvas.getHeight( ) );
		canvasGraphics.drawImage( image, 0, 0, this );
		canvas.drawImage( canvasImage, 0, 0, this );
		
		// Draw the "global position" image
		updateGlobalPosition( );
		
		// Update the zoom area
		updateZoomLevelArea( );
	}
	
	/** -- PRIVATE METHODS ------------------------------------------------- */
	
	/**
	 * Draw a fancy zoom area. Draw some smooth color transition
	 * from the left of the image to the right and draw the
	 * current zoom level as a vertical line.
	 */
	private void updateZoomLevelArea( )
	{
		if ( m_LastMapInfo == null || m_InitMapInfo == null ) return;
		
		BufferedImage image = new BufferedImage( 
		                         m_SmoothZoomCanvas.getWidth( ), 
		                         m_SmoothZoomCanvas.getHeight( ),
		                         BufferedImage.TYPE_INT_RGB );
		                         
		Graphics g = image.getGraphics( );
		                         
		for ( int i = 0; i < image.getWidth( ); i++ ) {
			int iColor  = Math.round( ( ( float ) 255 / image.getWidth( ) ) * i );
			Color c 		= new Color( iColor, 0, 0 );
			g.setColor( c );
			g.fillRect( i, 0, 1, image.getHeight( ) );	
		}
		
		// Draw the current level line
		int iCurrentZoomLevelOffset = Math.round( ( m_LastMapInfo.getZoomLevel( ) / 
		                                           ( float ) m_InitMapInfo.getZoomLevels( ) ) * 
		                                           image.getWidth( ) );
		int iCurrentZoomLevelWidth	= image.getWidth( ) / m_InitMapInfo.getZoomLevels( );
		if ( iCurrentZoomLevelWidth == 0 ) iCurrentZoomLevelWidth = 1;
		
		g.setColor( Color.YELLOW );
		g.fillRect( iCurrentZoomLevelOffset, 0, iCurrentZoomLevelWidth, image.getHeight( ) );
		
		m_SmoothZoomCanvas.getGraphics( ).drawImage( image, 0, 0, this );
	}
	
	/**
	 * Updates the canvas that shows the current position of the viewer
	 * on the map. The current zoom level map width/height are scaled
	 * to the size of the canvas. Same for the viewer rectangle dimensions.
	 */
	private void updateGlobalPosition( )
	{
		if ( m_LastMapInfo == null ) return;
		// Get the bigger dimension of the map and scale it against 
		// the appropriate side of the canvas, depending on which
		// side is that.
		int iCanvasWidth	= m_InfoCanvas.getWidth( );
		int iCanvasHeight	= m_InfoCanvas.getHeight( );
		int iMapWidth		= m_LastMapInfo.getMapWidth( );
		int iMapHeight		= m_LastMapInfo.getMapHeight( );
		int iViewerX		= m_LastMapInfo.getViewerX( );
		int iViewerY		= m_LastMapInfo.getViewerY( );
		int iViewerWidth	= m_LastMapInfo.getViewerWidth( );
		int iViewerHeight	= m_LastMapInfo.getViewerHeight( );
		
		float fScaleRatio = 1;
		if ( iMapWidth >= iMapHeight ) {
			fScaleRatio = iCanvasWidth / ( float ) iMapWidth;
		} else {
			fScaleRatio = iCanvasHeight / ( float ) iMapHeight; 
		}
		
		int iScaledMapWidth		= Math.round( iMapWidth  * fScaleRatio );
		int iScaledMapHeight		= Math.round( iMapHeight * fScaleRatio );
		int iScaledViewerX		= ( iScaledMapWidth * iViewerX ) / iMapWidth;
		int iScaledViewerY		= ( iScaledMapHeight * iViewerY ) / iMapHeight; 
		int iScaledViewerWidth	= Math.round( iViewerWidth * fScaleRatio );
		int iScaledViewerHeight	= Math.round( iViewerHeight * fScaleRatio );
		
		BufferedImage image = new BufferedImage( iCanvasWidth, iCanvasHeight, BufferedImage.TYPE_INT_RGB );
		Graphics g = image.getGraphics( );
		
		// Draw the background color
		g.setColor( Color.GRAY );
		g.fillRect( 0, 0, iScaledMapWidth, iScaledMapHeight );
		
		// Draw a nice border around it
		g.setColor( Color.BLACK );
		g.drawRect( 0, 0, iScaledMapWidth - 1, iScaledMapHeight - 1 );
		
		// Draw the viewer rectangle with a red border around
		// with a white fill.
		g.setColor( Color.WHITE );
		g.fillRect( iScaledViewerX, iScaledViewerY, iScaledViewerWidth, iScaledViewerHeight );
		g.setColor( Color.RED );
		g.drawRect( iScaledViewerX, iScaledViewerY, iScaledViewerWidth - 1, iScaledViewerHeight - 1 );
		
		m_InfoCanvas.getGraphics( ).drawImage( image, 0, 0, this );
	}
	
	/**
	 * Sets the current view of the toolbar
	 */
	private void setView( String szView )
	{
		CardLayout layout = ( CardLayout ) m_ToolbarContentPanel.getLayout( );
		layout.show( m_ToolbarContentPanel, szView );
		m_szCurrentView = szView;
		
		// Set the enable/disable state of the button which
		// view is currently on
		m_ViewSearchButton.setEnabled( true );
		m_ViewAreasButton.setEnabled( true );
		m_SaveMapButton.setVisible( false );
		m_LogoutButton.setVisible( false );
		
		setInfo( "" );
		
		if ( szView.equals( TOOLBAR_VIEW_SEARCH ) ) {
			m_ViewSearchButton.setEnabled( false );
			setInfo( "You are in SEARCH mode" );
		} else if ( szView.equals( TOOLBAR_VIEW_AREAS ) ) {
			setInfo( "You are in EDIT AREAS mode" );
			m_SaveMapButton.setVisible( true );
			m_LogoutButton.setVisible( true );
			m_ViewAreasButton.setEnabled( false );
		} else if ( szView.equals( TOOLBAR_VIEW_LOGIN ) ) {
			setInfo( "You are in LOGIN mode" );
		}
		
		repaint( );
	}
	
	/**
	 * Sets a message in the info label
	 * @param szMessage
	 */
	private void setInfo( String szMessage )
	{
		m_InfoLabel.setToolTipText( szMessage );
		m_InfoLabel.setText( szMessage );
	}
	
	/**
	 * Initializes the base structures used in the applet:
	 * - Map request
	 * - Map image
	 * - Map info
	 */	
	private void load( )
	{
		// Set the current view
		setView( TOOLBAR_VIEW_SEARCH );
		setAreaGUIState( );
		resetAreaFields( );
		
		try {
			Hashtable urls	= new Hashtable( );
			String szBaseURL = getParameter( "BaseServletPath" );
			if ( szBaseURL == null ) szBaseURL = "http://localhost:8080/DynaMap";
			if ( szBaseURL.charAt( szBaseURL.length( )  - 1 ) != '/' ) szBaseURL += '/';
			// INIT URL
			urls.put( new Integer( MapRequest.URL_TYPE_INIT 		), szBaseURL + "ShowMap" );
			//	SHOW MAP URL
			urls.put( new Integer( MapRequest.URL_TYPE_SHOW_MAP 	), szBaseURL + "ShowMap" );
			// ADD AREA URL
			urls.put( new Integer( MapRequest.URL_TYPE_ADD_AREA 	), szBaseURL + "AddArea" );
			// UPDATE AREA URL
			urls.put( new Integer( MapRequest.URL_TYPE_UPDATE_AREA ),szBaseURL + "UpdateArea" );
			// REMOVE AREA URL
			urls.put( new Integer( MapRequest.URL_TYPE_REMOVE_AREA ),szBaseURL + "RemoveArea" );
			// INFO URL
			urls.put( new Integer( MapRequest.URL_TYPE_INFO 		), szBaseURL + "GetInfo" );
			// SAVE MAP URL
			urls.put( new Integer( MapRequest.URL_TYPE_SAVE_MAP   ), szBaseURL + "SaveMap" );
			// SEARCH MAP URL
			urls.put( new Integer( MapRequest.URL_TYPE_SEARCH_MAP ), szBaseURL + "Search"  );
			// LOGIN URL
			urls.put( new Integer( MapRequest.URL_TYPE_LOGIN      ), szBaseURL + "Login"   );
			// LOGOUT URL
			urls.put( new Integer( MapRequest.URL_TYPE_LOGOUT     ), szBaseURL + "Logout"   );
			
			m_MapRequest	= new MapRequest( urls );
			mapAction( MAP_ACTION_INIT, null );
			repaint( );
		} catch ( MapRequestException e ) {
			System.out.println( "Error: " + e.getMessage( ) );
		}
	}

	
	/**
	 * Identifies if a map action has already begun
	 * 
	 * @return true if a map action can be started, false if another action is
	 *         already running
	 */
	private boolean mapActionBegin( )
	{
		if ( m_bMapAction == false ) {
			m_bMapAction = true;
			return true;
		} 
		return false;
	}
	
	/**
	 * Sets a flag that a map action has finished
	 */
	private void mapActionEnd( )
	{
		m_bMapAction = false;
	}
	
	/**
	 * Invokes a map request depending on the action type
	 * 
	 * @param iActionType - the type of action (see MAP_ACTION*)
	 * @param params - a hashtable of key-value parameter name-value
	 *                 pairs. Both name and value are String objects.
	 */
	private void mapAction( int iActionType, Hashtable params )
	{
		if ( mapActionBegin( ) ) {
			try {
				showStatus( "Loading, please wait..." );
				System.out.println( "Map action: Action type: " + iActionType );
				switch ( iActionType ) {
					case MAP_ACTION_INIT:
						String szMapID		= params == null ? null : ( String )params.get( Constants.PARAM_MAP_ID );
						int iMapID			= szMapID != null ? Integer.parseInt( szMapID ) : 
														m_InitMapInfo == null ? -1 : m_InitMapInfo.getMapID( );
						m_MapImage 			= m_MapRequest.initMapRequest( m_Canvas.getWidth( ), 
						                                                 m_Canvas.getHeight( ),
						                                                 iMapID );
						m_InitMapInfo  	= m_MapRequest.infoMapRequest( Constants.INFO_TYPE_INIT, null );
						m_LastMapInfo		= m_InitMapInfo;
						fillMapsList( );
						fillIconsList( m_SearchIconsComboBox );
						initAreasPropertySheet( );
						break;
					case MAP_ACTION_ZOOM_IN:
						m_MapImage 		= m_MapRequest.zoomMapRequest( Constants.EVENT_ON_ZOOM_IN, null );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_ZOOM_OUT:
						m_MapImage 		= m_MapRequest.zoomMapRequest( Constants.EVENT_ON_ZOOM_OUT, null );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_ZOOM_TO:
						m_MapImage 		= m_MapRequest.zoomMapRequest( Constants.EVENT_ON_ZOOM_TO, params );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_MOVE_LEFT:
						m_MapImage 		= m_MapRequest.moveMapRequest( Constants.EVENT_ON_MOVE_LEFT, null );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_MOVE_UP:
						m_MapImage 		= m_MapRequest.moveMapRequest( Constants.EVENT_ON_MOVE_UP, null );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_MOVE_RIGHT:
						m_MapImage 		= m_MapRequest.moveMapRequest( Constants.EVENT_ON_MOVE_RIGHT, null );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_MOVE_DOWN:
						m_MapImage 		= m_MapRequest.moveMapRequest( Constants.EVENT_ON_MOVE_DOWN, null );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_ZOOM_IN_MOVE:
						m_MapImage 		= m_MapRequest.combinedMapRequest( Constants.EVENT_ON_ZOOM_IN_MOVE, params );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_ZOOM_OUT_MOVE:
						m_MapImage 		= m_MapRequest.combinedMapRequest( Constants.EVENT_ON_ZOOM_OUT_MOVE, params );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;				
					case MAP_ACTION_ADD_AREA:
						m_MapImage 		= m_MapRequest.addArea( params );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_UPDATE_AREA:
						m_MapImage 		= m_MapRequest.updateArea( params );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_REMOVE_AREA:	
						m_MapImage 		= m_MapRequest.removeArea( params );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_GET_AREA:
						MapRequestInfo mapInfo = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_AREA, params );
						// Get the first area
						AreaInfo areaInfo = mapInfo.getFirstArea( );
						if ( areaInfo != null) {
							m_AreaView.setAreaID( areaInfo.id );
							fillAreaFields( areaInfo );
						} else {
							System.out.println( "Map action: Nothing in the box" );
						}
						break;			
					case MAP_ACTION_SAVE_MAP:
						m_MapImage 		= m_MapRequest.saveMap( params );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						break;
					case MAP_ACTION_LOGOUT:
						m_LastMapInfo	= m_MapRequest.logoutRequest( );
						setView( TOOLBAR_VIEW_SEARCH );
						break;
					case MAP_ACTION_CLICK:
						mapInfo = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_AREA, params );
						// Open the URL of the area (if any)
						areaInfo = mapInfo.getFirstArea( );
						if ( areaInfo != null ) {
							String szURL = ( String ) areaInfo.info.get( Constants.AREA_ATTRIBUTE_URL );
							if ( szURL != null ) {
								getAppletContext().showDocument( new URL( szURL ), "_blank" );
							}
						} 
						break;		
					case MAP_ACTION_SEARCH_MAP:
						Hashtable searchParams = getSearchParams( );
						m_MapImage 		= m_MapRequest.searchMapRequest( searchParams );
						m_LastMapInfo  = m_MapRequest.infoMapRequest( Constants.INFO_TYPE_COMMON, null );
						setShortestPathInfoFields( );
						break;		
					case MAP_ACTION_LOGIN:
						params = new Hashtable( );
						params.put( Constants.PARAM_LOGIN	, m_LoginTextField.getText( ) );
						params.put( Constants.PARAM_PASSWORD, m_PasswordTextField.getText( ) );
						m_LastMapInfo = m_MapRequest.loginRequest( params );
						if ( m_LastMapInfo.isLogged( ) ) {
							setView( TOOLBAR_VIEW_AREAS );
						}
						m_LoginTextField.setText( "" );
						m_PasswordTextField.setText( "" );
						break;
				}
				showStatus( "Done." );
				
				mapActionEnd( );
				repaint( );
			} catch ( MapRequestException e ) {
				System.out.println( "Failed executing init: " + e.getMessage( ) );
				mapActionEnd( );
			} catch ( MalformedURLException e ) {
				System.out.println( "Malformed URL: " + e.getMessage( ) );
				mapActionEnd( );
			}
		} else {
			showStatus( "Another action is taking place, please wait..." );
		}
	}
	
	/**
	 * Checks the list of current areas over the map and if
	 * there's an area that is below the given (x, y)
	 * position, it is returned 
	 * 
	 * @param x - the X position
	 * @param y - the Y position
	 * 
	 * @return null if (x, y) the point is not over any map
	 *         or AreaInfo object if it is
	 */
	private AreaInfo getAreaOnPos( int x, int y )
	{
		Enumeration areas = m_LastMapInfo.getAreas( );
		while ( areas.hasMoreElements( ) ) {
			AreaInfo area = ( AreaInfo )areas.nextElement( );
			//showStatus( x + ", " + y + " vs. " + area.x + ", " + area.y + ", " + area.width + ", " + area.height );
			if ( x >= area.x && x <= area.x + area.width &&
				  y >= area.y && y <= area.y + area.height ) {
				return area;
			}
		}
		return null;
	}
		
	/**
	 * Fills the maps list combo box using the information
	 * from the initialization of the map
	 */
	private void fillMapsList( )
	{
		Enumeration maps = m_InitMapInfo.getMaps( );
		
		// Add the initial map definition 
		MapDefinition initialMapDefinition 	= new MapDefinition( );
		initialMapDefinition.id 				= -1;
		initialMapDefinition.name				= "[Default map]";
		
		m_MapChoice.removeAllItems( );
		m_MapChoice.addItem( initialMapDefinition );
		while ( maps.hasMoreElements( ) ) {
			MapDefinition mapDefinition = ( MapDefinition ) maps.nextElement( );
			m_MapChoice.addItem( mapDefinition );
		}
	}
	
	/**
	 * Fills the list of currently available icons in a given combo
	 * 
	 * @param combo - the JComboBox object
	 */
	private void fillIconsList( JComboBox combo )
	{
		Enumeration groups = m_InitMapInfo.getGroups( );
		combo.removeAllItems( );
		
		ComboBoxCategoryItem item = new ComboBoxCategoryItem( ComboBoxCategoryItem.TYPE_NONE, null );
		combo.addItem( item );
		
		while ( groups.hasMoreElements( ) ) {
			GroupInfo groupInfo = ( GroupInfo ) groups.nextElement( );
			item = new ComboBoxCategoryItem( ComboBoxCategoryItem.TYPE_GROUP, groupInfo );
			combo.addItem( item );
			
			Enumeration iconIDs = groupInfo.icons.keys( );
			while ( iconIDs.hasMoreElements( ) ) {
				IconInfo iconInfo = ( IconInfo )groupInfo.icons.get( iconIDs.nextElement( ) );
				item = new ComboBoxCategoryItem( ComboBoxCategoryItem.TYPE_ICON, iconInfo );
				combo.addItem( item );
			}
		}		
	}
	
	/**
	 * Fills the areas property sheet:
	 * - combox box that must display icons when adding/updating
	 * - the combo box for selecting a sub-map
	 * and area
	 */
	private void initAreasPropertySheet( )
	{
		fillIconsList( m_AreaIconComboBox );
		Enumeration maps = m_InitMapInfo.getMaps( );
	
		// Add the initial map definition 
		MapDefinition initialMapDefinition 	= new MapDefinition( );
		initialMapDefinition.id 				= 0;
		initialMapDefinition.name				= "-------";
	
		m_SubMapComboBox.removeAllItems( );
		m_SubMapComboBox.addItem( initialMapDefinition );
		while ( maps.hasMoreElements( ) ) {
			MapDefinition mapDefinition = ( MapDefinition ) maps.nextElement( );
			m_SubMapComboBox.addItem( mapDefinition );
		}
	}
	
	/**
	 * Fills the area coordinates fields with the given coordinates
	 * 
	 * @param x - the x value
	 * @param y - the y value
	 */
	private void setAreaCoorFields( int x, int y )
	{
		m_AreaXTextField.setText( "" + x );
		m_AreaYTextField.setText( "" + y );
	}
	
	/**
	 * Resets the area fields
	 */
	private void resetAreaFields( )
	{
		m_AreaXTextField.setText( "" );
		m_AreaYTextField.setText( "" );
		m_AreaNameTextField.setText( "" );
		m_AreaDescTextField.setText( "" );
		m_AreaURLTextField.setText( "" );
		if ( m_AreaIconComboBox.getItemCount( ) > 0 ) {
			m_AreaIconComboBox.setSelectedIndex( 0 );
		}
		if ( m_SubMapComboBox.getItemCount( ) > 0 ) {
			m_SubMapComboBox.setSelectedIndex( 0 );
		}
	}
	
	/**
	 * Fills the area fields with the information found
	 * in the passed AreaInfo object
	 * 
	 * @param areaInfo - the area information
	 */
	private void fillAreaFields( AreaInfo areaInfo )
	{
		// Set the dimensions
		m_AreaXTextField.setText( "" + areaInfo.x );
		m_AreaYTextField.setText( "" + areaInfo.y );
		
		// Set the description and URL
		String szName  = ( String ) areaInfo.info.get( Constants.AREA_ATTRIBUTE_NAME );
		String szDesc 	= ( String ) areaInfo.info.get( Constants.AREA_ATTRIBUTE_DESCRIPTION );
		String szURL 	= ( String ) areaInfo.info.get( Constants.AREA_ATTRIBUTE_URL );
		if ( szName != null ) { m_AreaNameTextField.setText( szName ); }
		if ( szDesc != null ) { m_AreaDescTextField.setText( szDesc ); }
		if ( szURL  != null ) { m_AreaURLTextField.setText( szURL ); 	}
		
		// Set the icon
		String szIconID = ( String )areaInfo.info.get( Constants.AREA_ATTRIBUTE_ICON_ID );
		if ( szIconID != null ) {
			int iIconID = Integer.parseInt( szIconID );
			
			for ( int i = 0; i < m_AreaIconComboBox.getItemCount( ); i++ ) {
				ComboBoxCategoryItem item = ( ComboBoxCategoryItem ) m_AreaIconComboBox.getItemAt( i );
				if ( item.getType( ) == ComboBoxCategoryItem.TYPE_ICON ) {
					IconInfo iconInfo = ( IconInfo ) item.getInfo( );
					if ( iIconID == iconInfo.id ) {
						m_AreaIconComboBox.setSelectedIndex( i );
						break;
					}
				}
			}
		}
		
		// Set the sub-map
		String szSubMapID = ( String )areaInfo.info.get( Constants.AREA_ATTRIBUTE_SUB_MAP );
		if ( szSubMapID != null ) {
			int iSubMapID = Integer.parseInt( szSubMapID );
			Enumeration maps = m_InitMapInfo.getMaps( );
			while ( maps.hasMoreElements( ) ) {
				MapDefinition mapDefinition = ( MapDefinition ) maps.nextElement( );
				if ( mapDefinition.id == iSubMapID ) {
					m_SubMapComboBox.setSelectedItem( mapDefinition );
					break;
				}
			}
		}
	}
	
	/**
	 * Adds or updates an area depending on the AreaView.mode.
	 * If we're updating an area, the ID of the area that's
	 * updated is AreaView.areaID
	 * 
	 * @return true if everything with the action went fine
	 */
	private boolean clickOKAreaButton( )
	{
		// Fill a hashtable with the currently filled area properties
		Hashtable props = new Hashtable( );
		props.put( Constants.AREA_ATTRIBUTE_X				, m_AreaXTextField.getText( ) );
		props.put( Constants.AREA_ATTRIBUTE_Y				, m_AreaYTextField.getText( ) );
		props.put( Constants.AREA_ATTRIBUTE_WIDTH			, "" + m_InitMapInfo.getPreferredAreaWidth() );
		props.put( Constants.AREA_ATTRIBUTE_HEIGHT		, "" + m_InitMapInfo.getPreferredAreaHeight() );
		props.put( Constants.AREA_ATTRIBUTE_NAME        , m_AreaNameTextField.getText( ) );
		props.put( Constants.AREA_ATTRIBUTE_DESCRIPTION	, m_AreaDescTextField.getText( ) );
		props.put( Constants.AREA_ATTRIBUTE_URL			, m_AreaURLTextField.getText( ) );
		props.put( Constants.PARAM_MAP_ID					, "" + m_InitMapInfo.getMapID( ) );
		
		// Add the icon ID too
		ComboBoxCategoryItem categoryItem = ( ComboBoxCategoryItem ) m_AreaIconComboBox.getSelectedItem( );
		if ( categoryItem != null && categoryItem.getType( ) == ComboBoxCategoryItem.TYPE_ICON ) {
			IconInfo iconInfo = ( IconInfo ) categoryItem.getInfo( );
			props.put( Constants.AREA_ATTRIBUTE_ICON_ID, "" + iconInfo.id );
		}
		
		MapDefinition mapDefinition = ( MapDefinition ) m_SubMapComboBox.getSelectedItem( );
		if ( mapDefinition != null && mapDefinition.id > 0 ) {
			props.put( Constants.AREA_ATTRIBUTE_SUB_MAP, "" + mapDefinition.id );
		}
		
		switch ( m_AreaView.getMode( ) ) {
			case AreaView.AREA_VIEW_MODE_NEW:
				// Add area
				if( verifyAreaFields( ) == true ) {
					mapAction( MAP_ACTION_ADD_AREA, props );
				} else {
					return false;
				}
				break;
			case AreaView.AREA_VIEW_MODE_UPDATE:
				// Update an existing area
				if( verifyAreaFields( ) == true ) {
					if ( m_AreaView.getAreaID( ) > 0 ) {
						props.put( Constants.AREA_PARAM_UNIQUE_ID, "" + m_AreaView.getAreaID( ) );
						mapAction( MAP_ACTION_UPDATE_AREA, props );
					}
				} else {
					return false;
				}
				break;
			case AreaView.AREA_VIEW_MODE_REMOVE:
				if ( m_AreaView.getAreaID( ) > 0 ) {
					props.put( Constants.AREA_PARAM_UNIQUE_ID, "" + m_AreaView.getAreaID( ) );
					mapAction( MAP_ACTION_REMOVE_AREA, props );
				}
				break;
			default:
				// Do nothing
		}
		
		return true;
	}
	
	
	/**
	 * Sets the state of the controls in the GUI according
	 * to the current area mode
	 *
	 */
	private void setAreaGUIState( )
	{
		// If the view mode is not "none"
		// the controls must be always visible
		boolean bCtrlsVisibility = m_AreaView.getMode( ) == AreaView.AREA_VIEW_MODE_NONE ? 
		                           false : true;
		boolean bCtrlEnabled	= m_AreaView.getMode( ) == AreaView.AREA_VIEW_MODE_REMOVE ? 
		                         	false : true;

		m_AreaXTextField.getParent().setVisible( bCtrlsVisibility 	);
		
		m_AreaXTextField.setEnabled	( false 					);
		m_AreaYTextField.setEnabled	( false 					);
		m_AreaNameTextField.setEnabled( bCtrlEnabled       );
		m_AreaDescTextField.setEnabled( bCtrlEnabled 		);
		m_AreaURLTextField.setEnabled	( bCtrlEnabled 		);
		m_AreaIconComboBox.setEnabled	( bCtrlEnabled 		);
		
		switch ( m_AreaView.getMode( ) )	{
			case AreaView.AREA_VIEW_MODE_NONE:
				setInfo( "" );
				m_NewAreaButton.setEnabled( true );
				m_UpdateAreaButton.setEnabled( true );
				m_RemoveAreaButton.setEnabled( true );
				break;
			case AreaView.AREA_VIEW_MODE_NEW:
				setInfo( "You are in ADD NEW AREA mode" );
				m_NewAreaButton.setEnabled( false );
				m_UpdateAreaButton.setEnabled( true );
				m_RemoveAreaButton.setEnabled( true );
				break;
			case AreaView.AREA_VIEW_MODE_UPDATE:
				setInfo( "You are in UPDATE AREA mode" );
				m_NewAreaButton.setEnabled( true );
				m_UpdateAreaButton.setEnabled( false );
				m_RemoveAreaButton.setEnabled( true );
				break;
			case AreaView.AREA_VIEW_MODE_REMOVE:
				setInfo( "You are in REMOVE AREA mode" );
				m_NewAreaButton.setEnabled( true );
				m_UpdateAreaButton.setEnabled( true );
				m_RemoveAreaButton.setEnabled( false );
				break;
		}
	}
	
	/**
	 * Checks if all area fields - x, y, width, height, etc. 
	 * contain valid values
	 * 
	 * @return true if the field values are correct, false otherwise
	 */
	private boolean verifyAreaFields( )
	{
		String szX			= m_AreaXTextField.getText( ).trim( );
		String szY			= m_AreaYTextField.getText( ).trim( );
		String szName		= m_AreaNameTextField.getText( ).trim( );
		String szDesc		= m_AreaDescTextField.getText( ).trim( );
		String szURL		= m_AreaURLTextField.getText( ).trim( );
		ComboBoxCategoryItem category = ( ComboBoxCategoryItem ) m_AreaIconComboBox.getSelectedItem( );
		
		// Check if the right item is selected
		if ( category.getType( ) != ComboBoxCategoryItem.TYPE_ICON ) {
			MessageBox message = new MessageBox( this, MessageBox.TYPE_ERROR,
												 "Please select a category" ); 
			return false;
		}
		
		// Check coordinate fields
		if ( szX.equals( "" ) || szY.equals( "" ) ) {
			MessageBox message = new MessageBox( this, MessageBox.TYPE_ERROR,
			                                     "Please click somewhere on the screen " + 
			                                     "to place the area" ); 
			return false; 
		} 
		
		// Check the name 
		if ( szName.equals( "" ) ) {
			MessageBox message = new MessageBox( this, MessageBox.TYPE_ERROR,
			                                     "Please enter area name" );
			return false;
		}
		
		// Check description
		if ( szDesc.equals( "" ) ) { 
			MessageBox message = new MessageBox( this, MessageBox.TYPE_ERROR,
												 "Please enter area description" ); 
			return false; 
		}
		
		// Check URL existence
		if ( szURL.equals( "" ) ) {
			MessageBox message = new MessageBox( this, MessageBox.TYPE_ERROR,
												 "Please enter area URL" );
			return false; 
		}
		
		// Check if URL is valid
		String[] supportedProtocols = {
			"http://", "https://", "ftp://", "mailto:", "gopher://"
		};
		
		boolean bFoundProtocol = false;
		for ( int i = 0; i < supportedProtocols.length; i++ ) {
			if ( szURL.toLowerCase().indexOf( supportedProtocols[ i ] ) == 0 ) {
				bFoundProtocol = true;
				break;
			}
		}
		if ( !bFoundProtocol ) {
			MessageBox message = new MessageBox( this, MessageBox.TYPE_ERROR,
												 "Please enter a valid URL" );
			return false;
		}
		
		return true;
	}
	
	/**
	 * Merges two hashtables into a third one
	 * 
	 * @param h1 - the first hashtable
	 * @param h2 - the second hashtable
	 * 
	 * @return the merged result hashtable
	 */
	private Hashtable mergeParams( Hashtable h1, Hashtable h2 )
	{
		Hashtable result = new Hashtable( );
		Enumeration keys = h1.keys( );
		while ( keys.hasMoreElements( ) ) {
			Object key = keys.nextElement( );
			result.put( key, h1.get( key ) );
		}
		keys = h2.keys( );
		while ( keys.hasMoreElements( ) ) {
			Object key = keys.nextElement( );
			result.put( key, h2.get( key ) );
		}
		
		return result;
	}
	
	/**
	 * Returns a hashtable of key-value pairs reprsenting the
	 * current search panel state.
	 * 
	 * @return a hashtable with the parameters
	 */
	private Hashtable getSearchParams( )
	{
		Hashtable searchParams = new Hashtable( );
		searchParams.put( Constants.PARAM_SEARCH_QUERY, m_SearchTextField.getText( ) );
		
		// Get the areas that are used to search for the shortest path between them
		if ( m_SearchView.areAreasReady( ) && 
		     m_ToggleShowSPathCheckbox.isSelected( ) == true ) {
			AreaInfo[] areas = m_SearchView.getAreas( );
			searchParams.put( Constants.PARAM_SEARCH_AREA1, "" + areas[ 0 ].id );
			searchParams.put( Constants.PARAM_SEARCH_AREA2, "" + areas[ 1 ].id );
		}
		
		if ( m_ShowRoadsCheckbox.isSelected( ) ) {
			searchParams.put( Constants.PARAM_SHOW_GRAPH, "1" );
		}
		
		// Set the search attribute restrictions
		if ( m_SearchInNameCheckbox.isSelected( ) ) {
			searchParams.put( Constants.PARAM_SEARCH_IN_NAME, "1" );
		}
		if ( m_SearchInDescCheckbox.isSelected( ) ) {
			searchParams.put( Constants.PARAM_SEARCH_IN_DESCRIPTION, "1" );
		}
		if ( m_SearchInURLCheckbox.isSelected( ) ) {
			searchParams.put( Constants.PARAM_SEARCH_IN_URL, "1" );
		}
		
		// Check if there's an icon selected to search by
		ComboBoxCategoryItem selectedIcon = ( ComboBoxCategoryItem ) 
											m_SearchIconsComboBox.getSelectedItem( );
		if ( selectedIcon != null ) {
			String szIconIDs = "";
			// If this is a single icon - put the icon ID only. Otherwise
			// it's a group. In that case put all the icon IDs from the
			// group in a comma separated list
			if ( selectedIcon.getType( ) == ComboBoxCategoryItem.TYPE_ICON ) {
				IconInfo iconInfo = ( IconInfo ) selectedIcon.getInfo( );
				szIconIDs = "" + iconInfo.id; 
			} else if ( selectedIcon.getType( ) == ComboBoxCategoryItem.TYPE_GROUP ) {
				GroupInfo groupInfo = ( GroupInfo ) selectedIcon.getInfo( );
				Enumeration iconIDs = groupInfo.icons.keys( );
				while ( iconIDs.hasMoreElements( ) ) {
					IconInfo iconInfo = ( IconInfo ) groupInfo.icons.get( iconIDs.nextElement( ) );
					szIconIDs += iconInfo.id + ",";
				}
				if ( szIconIDs.length( ) > 0 ) szIconIDs = szIconIDs.substring( 0, szIconIDs.length( ) - 1 );
			}
			
			if ( szIconIDs.equals( "" ) == false ) { 
				searchParams.put( Constants.PARAM_SEARCH_ICON_IDS, szIconIDs );
			}
		}
		
		return searchParams;
	}

	/**
	 * Resets the search view controls and search view state
	 */
	private void resetSearch( )
	{
		m_SearchView.reset( );
		m_SearchTextField.setText( "" );
		m_ToggleShowSPathCheckbox.setSelected( true );
		m_SearchInNameCheckbox.setSelected( true );
		m_SearchInDescCheckbox.setSelected( true );
		m_SearchInURLCheckbox.setSelected( true );
		m_SearchIconsComboBox.setSelectedIndex( 0 );
		m_SPathX1TextField.setText( "" );
		m_SPathY1TextField.setText( "" );
		m_SPathX2TextField.setText( "" );
		m_SPathY2TextField.setText( "" );
		m_ShowRoadsCheckbox.setSelected( false );
	}
	
	/**
	 * Updates the text fields in the panel that shows the shortest
	 * path search points
	 */
	private void setShortestPathSearchControls( )
	{
		String szX1	= "N/A";
		String szY1 = "N/A";
		String szX2 = "N/A";
		String szY2 = "N/A";

		AreaInfo[] areas = m_SearchView.getAreas( );		
		if ( m_SearchView.areAreasReady( ) ) {
			// Set both coordinate pairs
			szX1 = "" + areas[ 0 ].x;
			szY1 = "" + areas[ 0 ].y;
			szX2 = "" + areas[ 1 ].x;
			szY2 = "" + areas[ 1 ].y;
		} else if ( m_SearchView.getReadAreasCount( ) == 1 ) {
			szX1 = "" + areas[ 0 ].x;
			szY1 = "" + areas[ 0 ].y;
		}
		
		m_SPathX1TextField.setText( szX1 );
		m_SPathY1TextField.setText( szY1 );
		m_SPathX2TextField.setText( szX2 );
		m_SPathY2TextField.setText( szY2 );
		
		if ( m_SearchView.getReadAreasCount() == 2 ) {
			// Work as a "Search button" is pressed
			mapAction( MAP_ACTION_SEARCH_MAP, null );
		}
	}

	/**
	 * Fills the fields with the result from the shortest path search
	 */
	private void setShortestPathInfoFields( )
	{
		float fLength = m_LastMapInfo.getRoadLength( );
		fLength /= 1000.0f;
		m_SPathLengthLabel.setText( "Road length is approximately " + fLength + "km" );
	}
	
	/**
	 * @class lib.applets
	 * 
	 * Implements a simple tooltip information storage and creating an image
	 * out of the text data
	 * 
	 * @author Tihomir Lazarov
	 * @date   Jun 19, 2005
	 */
	private class TooltipInfo
	{
		String name				= "";
		String description	= "";
		String url				= "";
		int x						= 0;
		int y						= 0;

		/**
		 * Checks if any of the text tooltip information is set
		 *  
		 * @return true if it is set, false otherwise
		 */		
		public boolean isSet( ) 
		{
			return ( name != "" || description != "" || url != "" );		 
		}
		
		/**
		 * Resets the current tooltip information
		 */
		public void reset( )
		{
			name			= "";
			description	= "";
			url			= "";
			x				= 0;
			y				= 0;
		}
		
		/**
		 * Creates an image with some tooltip-like message
		 * 
		 * @param fm - the applet font metrics object
		 * 
		 * @return an object of type BufferedImage
		 */
		public BufferedImage createImage( FontMetrics fm )
		{
			String szName = name == "" ? "(Untitled)" : name;
			String szDesc = description == "" ? "(No description)" : description;
			String szURL  = url;
			
			int iOffset = 5;
			int iHeight	= fm.getHeight( ) * 3 + 3 * iOffset;
			int iWidth	= 150;
			
			BufferedImage image 	= new BufferedImage( iWidth, iHeight, BufferedImage.TYPE_INT_RGB );
			Graphics g 				= image.getGraphics( );
			g.setColor( Color.WHITE );
			g.fillRect( 0, 0, iWidth, iHeight );
			g.setColor( Color.BLACK );
			g.drawRect( 0, 0, iWidth - 1, iHeight - 1 );
			g.setColor( Color.BLACK );
			
			
			g.drawString( szName, iOffset, iOffset + fm.getHeight( ) * 1 );
			g.drawString( szDesc, iOffset, iOffset + fm.getHeight( ) * 2 );
			g.drawString( szURL , iOffset, iOffset + fm.getHeight( ) * 3 );
			
			return image;
		}
	}	
	
	private class ComboBoxCategoryItem
	{
		public static final int TYPE_NONE	= 0;
		public static final int TYPE_ICON	= 1;
		public static final int TYPE_GROUP	= 2;
		
		Object m_Info  = null;
		int m_iType		= TYPE_NONE;
		
		public ComboBoxCategoryItem( int iType, Object info )
		{
			m_Info 	= info;
			m_iType	= iType; 
		}
		
		public int getType( )
		{
			return m_iType;
		}
		
		public Object getInfo( )
		{
			return m_Info;
		}
		
		public String toString( )
		{
			switch ( m_iType ) {
				case TYPE_ICON:
					IconInfo iconInfo = ( IconInfo ) m_Info;
					return "  *** " + iconInfo.description;
				case TYPE_GROUP:
					GroupInfo groupInfo = ( GroupInfo ) m_Info;
					return "+ [" + groupInfo.name.toUpperCase( ) + "]";
				default:
					return "-------";
			}	
		}	
	}
}
