package lib.applets;

import javax.swing.*;

import java.awt.Font;
import java.awt.Component;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.CardLayout;

/**
 * @class lib.applets
 * 
 * Creates the GUI of the client-side map
 * 
 * @author root
 * @date   Jun 7, 2005
 */
public class MapGUI extends JApplet 
{
	static final String TOOLBAR_VIEW_NONE		= "ToolbarViewNone";
	static final String TOOLBAR_VIEW_SEARCH	= "ToolbarViewSearch";
	static final String TOOLBAR_VIEW_AREAS		= "ToolbarViewAreas";
	static final String TOOLBAR_VIEW_LOGIN		= "ToolbarViewLogin";
	
	// MAP + NAVIGATION panel elements
	JButton m_MoveLeftButton				= new JButton( "<" );
	JButton m_MoveRightButton				= new JButton( ">" );
	JButton m_MoveUpButton					= new JButton( "/\\" );
	JButton m_MoveDownButton				= new JButton( "\\/" );
	JLabel m_Canvas							= new JLabel( " " );

	// SEARCH panel elements
	JTextField m_SearchTextField 			= new JTextField( "", 15 );
	JButton m_SearchButton					= new JButton( "OK" );
	JButton m_SearchCancelButton			= new JButton( "Cancel" );
	JButton m_ResetButton					= new JButton( "Reset" );
	JCheckBox m_ToggleShowSPathCheckbox = new JCheckBox( "Show shortest path", true );
	JCheckBox m_ShowRoadsCheckbox			= new JCheckBox( "Show roads", false );
	JCheckBox m_SearchInNameCheckbox		= new JCheckBox( "Name", true );
	JCheckBox m_SearchInDescCheckbox		= new JCheckBox( "Description", true );
	JCheckBox m_SearchInURLCheckbox		= new JCheckBox( "URL", true );
	JComboBox m_SearchIconsComboBox  	= new JComboBox( );
	JTextField m_SPathX1TextField			= new JTextField( "", 3 );
	JTextField m_SPathY1TextField			= new JTextField( "", 3 );
	JTextField m_SPathX2TextField			= new JTextField( "", 3 );
	JTextField m_SPathY2TextField			= new JTextField( "", 3 );
	JLabel m_SPathLengthLabel				= new JLabel( "" );
	
	// SELECTOR panel elements
	JLabel m_SelectMapLabel					= new JLabel( "Map" );
	JComboBox m_MapChoice					= new JComboBox( );
	JButton m_SaveMapButton					= new JButton( "Save" );
	JButton m_LogoutButton					= new JButton( "Logout" );
	JButton m_ZoomInButton					= new JButton( "+" );
	JButton m_ZoomOutButton					= new JButton( "-" );
	JLabel m_SmoothZoomCanvas				= new JLabel( "                           " );
	
	// VIEWS PANEL
	JPanel m_ToolbarContentPanel 			= new JPanel( );
	JButton m_ViewSearchButton				= new JButton( "Search" );
	JButton m_ViewAreasButton				= new JButton( "Areas" );
	
	// AREAS PANEL
	JButton m_NewAreaButton					= new JButton( "New" );
	JButton m_UpdateAreaButton				= new JButton( "Update" );
	JButton m_RemoveAreaButton				= new JButton( "Remove" );
	JButton m_OKAreaButton					= new JButton( "OK" );
	JButton m_CancelAreaButton				= new JButton( "Cancel" );
	JTextField m_AreaXTextField			= new JTextField( 3 );
	JTextField m_AreaYTextField			= new JTextField( 3 );
	JTextField m_AreaNameTextField		= new JTextField( 15 );
	JTextField m_AreaDescTextField		= new JTextField( 15 );
	JTextField m_AreaURLTextField			= new JTextField( 15 );
	JComboBox m_AreaIconComboBox			= new JComboBox( );
	JComboBox m_SubMapComboBox				= new JComboBox( );
	
	// LOGIN PANEL
	JTextField m_LoginTextField			= new JTextField( 15 );
	JTextField m_PasswordTextField		= new JPasswordField( 15 );
	JButton m_OKAuthButton					= new JButton( "Login" );
	JButton m_CancelAuthButton				= new JButton( "Cancel" );
	
	// INFO PANEL
	JLabel m_InfoLabel						= new JLabel( "" );
	JLabel m_InfoCanvas						= new JLabel( );
	
	/**
	 * Initializes the applet controls
	 */	
	public void init( )
	{
		super.init( );
		
		//////////////////////////////////////////////
		// DRAW PANELS
		//////////////////////////////////////////////
		//setSize( 900, 500 );
		getContentPane( ).setLayout( new GridLayout( ) );
		
		setFont( new Font( "Helvetica", Font.PLAIN, 10 ) );

		JPanel toolbarPanel		= arrangeToolbarPanel( );
		JPanel mapControlsPanel	= arrangeMapControlsPanel( );
		JPanel infoPanel			= arrangeInfoPanel( );
		
		JPanel toolbarInfoPanel	= new JPanel( new BorderLayout( ) );
		toolbarInfoPanel.add( BorderLayout.NORTH, toolbarPanel );
		toolbarInfoPanel.add( BorderLayout.CENTER, infoPanel );

		/////////////////////////////////////////
		// PUT THE PANELS TOGETHER
		/////////////////////////////////////////
		GridBagLayout baseLayout 		= new GridBagLayout( );
		JPanel basePanel 					= new JPanel( baseLayout );
		GridBagConstraints constraints = new GridBagConstraints( );
		
		constraints.fill			= GridBagConstraints.BOTH;
		
		// BASE PANEL
		constraints.gridheight	= GridBagConstraints.RELATIVE;
		constraints.gridwidth	= GridBagConstraints.REMAINDER;
		constraints.weighty		= 1.0;
		constraints.weightx		= 1.0;
		constraints.gridwidth	= GridBagConstraints.RELATIVE;
		baseLayout.setConstraints( mapControlsPanel, constraints );
		basePanel.add( mapControlsPanel );
		
		constraints.weightx		= 0.0;
		constraints.gridwidth	= GridBagConstraints.REMAINDER;
		baseLayout.setConstraints( toolbarInfoPanel, constraints );
		basePanel.add( toolbarInfoPanel );

		getContentPane( ).add( basePanel );
		
		setVisible( true );
	}
	
	/**
	 * Creates the map controls panel
	 * 
	 * @return the map controls panel
	 */
	private JPanel arrangeMapControlsPanel( )
	{
		// MAP NAVIGATION PANEL		
		JPanel mapNavPanel = new JPanel( );
		mapNavPanel.setLayout( new BorderLayout( ) );
		mapNavPanel.add( BorderLayout.WEST,   m_MoveLeftButton );
		mapNavPanel.add( BorderLayout.NORTH,  m_MoveUpButton );
		mapNavPanel.add( BorderLayout.EAST,   m_MoveRightButton );
		mapNavPanel.add( BorderLayout.SOUTH,  m_MoveDownButton );
		mapNavPanel.add( BorderLayout.CENTER, m_Canvas );
		
		// CHOICE PANEL
		JPanel choicePanel = new JPanel( );
		choicePanel.setLayout( new FlowLayout( ) );
		choicePanel.add( m_SelectMapLabel );
		choicePanel.add( m_MapChoice );
		m_SaveMapButton.setVisible( false );
		m_LogoutButton.setVisible( false );
		choicePanel.add( m_SaveMapButton );
		choicePanel.add( m_LogoutButton );
		
		// ZOOM PANEL
		JPanel zoomPanel = new JPanel( );
		zoomPanel.setLayout( new FlowLayout( ) );
		zoomPanel.add( m_ZoomOutButton );
		zoomPanel.add( m_SmoothZoomCanvas );
		zoomPanel.add( m_ZoomInButton );
		
		// SELECTOR PANEL
		JPanel selectorPanel = new JPanel( );
		selectorPanel.setLayout( new BorderLayout( ) );
		selectorPanel.add( BorderLayout.WEST, choicePanel );
		selectorPanel.add( BorderLayout.EAST, zoomPanel );
		
		//	CONTROL PANEL
		JPanel controlPanel = new JPanel( );
		controlPanel.setLayout( new BorderLayout( ) );
		controlPanel.add( BorderLayout.SOUTH, selectorPanel );
		
		// MAP CONTROLS PANEL
		GridBagLayout mapControlsLayout = new GridBagLayout( );
		JPanel mapControlsPanel = new JPanel( );
		mapControlsPanel.setLayout( mapControlsLayout );
		GridBagConstraints constraints = new GridBagConstraints( );
		
		constraints.fill 			= GridBagConstraints.BOTH;
		constraints.gridheight	= GridBagConstraints.RELATIVE;
		constraints.gridwidth	= GridBagConstraints.REMAINDER;
		constraints.weightx    	= 0.0;
		mapControlsLayout.setConstraints( controlPanel, constraints );
		mapControlsPanel.add( controlPanel );
		
		constraints.weightx 		= 1.0;
		constraints.weighty 		= 1.0;
		constraints.gridheight 	= GridBagConstraints.REMAINDER;
		mapControlsLayout.setConstraints( mapNavPanel, constraints );
		mapControlsPanel.add( mapNavPanel );

		return mapControlsPanel;		
	}
	
	/**
	 * Arranges the toolbar panel
	 * 
	 * @return the toolbar panel
	 */
	private JPanel arrangeToolbarPanel( )
	{
		// SEARCH PANEL
		JPanel searchPanel = new JPanel( );
		searchPanel.setLayout( new FlowLayout( ) );
		arrangeSearchPanel( searchPanel );
		
		// VIEWS PANEL
		JPanel viewsPanel	= new JPanel( new FlowLayout( ) );
		viewsPanel.add( m_ViewSearchButton );
		viewsPanel.add( m_ViewAreasButton );
		
		// TOOLBAR HEADER PANEL
		JPanel toolbarHeaderPanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
		toolbarHeaderPanel.add( viewsPanel );

		// TOOLBAR CONTENT PANEL		
		m_ToolbarContentPanel = new JPanel( new CardLayout( ) );
		m_ToolbarContentPanel.add( new JPanel( ) /* Empty */, TOOLBAR_VIEW_NONE );
		m_ToolbarContentPanel.add( searchPanel         , TOOLBAR_VIEW_SEARCH );
		m_ToolbarContentPanel.add( arrangeAreasPanel( ), TOOLBAR_VIEW_AREAS );
		m_ToolbarContentPanel.add( arrangeLoginPanel( ), TOOLBAR_VIEW_LOGIN );
		
		// TOOLBAR PANEL
		GridBagLayout toolbarLayout = new GridBagLayout( );
		GridBagConstraints constraints = new GridBagConstraints( );
		JPanel toolbarPanel = new JPanel( toolbarLayout );
		
		constraints.fill 			= GridBagConstraints.BOTH;
		constraints.gridwidth	= GridBagConstraints.REMAINDER;
		constraints.gridheight	= GridBagConstraints.RELATIVE;
		constraints.weightx		= 1.0;
		constraints.weighty		= 0.0;
		toolbarLayout.setConstraints( toolbarHeaderPanel, constraints );
		toolbarPanel.add( toolbarHeaderPanel );
		
		constraints.weightx		= 0.0;
		constraints.weighty		= 1.0;
		constraints.gridheight	= GridBagConstraints.REMAINDER;
		toolbarLayout.setConstraints( m_ToolbarContentPanel, constraints );
		toolbarPanel.add( m_ToolbarContentPanel );		
		
		return toolbarPanel;
	}
	
	/**
	 * Arranges the areas panel
	 * 
	 * @return the areas panel
	 */
	private JPanel arrangeAreasPanel( )
	{
		// AREAS PANEL
		JPanel areasHeaderPanel = new JPanel( new FlowLayout( ) );
		areasHeaderPanel.add( m_NewAreaButton );
		areasHeaderPanel.add( m_UpdateAreaButton );
		areasHeaderPanel.add( m_RemoveAreaButton );
		
		// AREA PROPERTIES PANEL
		JPanel areaContentPanel = new JPanel( new BorderLayout( ) );
		areaContentPanel.add( BorderLayout.NORTH, arrangeAreaPropertiesPanel( ) );
		
		JPanel areasPanel = new JPanel( );
		areasPanel.setLayout( new BorderLayout( ) );
		areasPanel.setAlignmentY( Component.TOP_ALIGNMENT );
		areasPanel.add( BorderLayout.NORTH, areasHeaderPanel );
		areasPanel.add( areaContentPanel );
		
		return areasPanel;
	}
	
	/**
	 * Arranges the area properties panel
	 * 
	 * @return the area properties panel
	 */
	private JPanel arrangeAreaPropertiesPanel( )
	{
		GridBagLayout layout	= new GridBagLayout( );
		GridBagConstraints cons = new GridBagConstraints( );
		JPanel areaPropertiesPanel = new JPanel( layout );
		areaPropertiesPanel.setAlignmentY( Component.TOP_ALIGNMENT );

		addPropertyField( areaPropertiesPanel, "X", m_AreaXTextField );
		addPropertyField( areaPropertiesPanel, "Y", m_AreaYTextField );
		addPropertyField( areaPropertiesPanel, "Name", m_AreaNameTextField );
		addPropertyField( areaPropertiesPanel, "Details", m_AreaDescTextField );
		addPropertyField( areaPropertiesPanel, "URL", m_AreaURLTextField );
		addPropertyField( areaPropertiesPanel, "Category", m_AreaIconComboBox );
		addPropertyField( areaPropertiesPanel, "Sub map", m_SubMapComboBox );
		
		JPanel okCancelButtonsPanel = new JPanel( new FlowLayout( ) );
		okCancelButtonsPanel.add( m_OKAreaButton );
		okCancelButtonsPanel.add( m_CancelAreaButton );
		addPropertyField( areaPropertiesPanel, "", okCancelButtonsPanel );
		
		areaPropertiesPanel.setVisible( false );
		return areaPropertiesPanel;
	}

	/**
	 * Adds a property field to a panel
	 * 
	 * @param panel - the panel
	 * @param szLabel - the label of the field
	 * @param component - the component to be added
	 */	
	private void addPropertyField( JPanel panel, String szLabel, Component component )
	{
		GridBagConstraints cons = new GridBagConstraints( );
		GridBagLayout layout    = ( GridBagLayout ) panel.getLayout( );
		
		cons.fill 		= GridBagConstraints.BOTH;
		
		cons.weightx	= 0.0;
		cons.gridwidth	= GridBagConstraints.RELATIVE;
		JLabel label	= new JLabel( szLabel );
		layout.setConstraints( label, cons ); 
		panel.add( label );
		
		cons.weightx	= 1.0;
		cons.gridwidth	= GridBagConstraints.REMAINDER;
		layout.setConstraints( component, cons );
		panel.add( component );
	}
	

	/**
	 * Arranges the search panel controls
	 * 
	 * @param searchPanel - the base search panel
	 */	
	private void arrangeSearchPanel( JPanel searchPanel )
	{
		JPanel base = new JPanel( new GridBagLayout( ) );
		JPanel simpleSearchPanel = new JPanel( );
		simpleSearchPanel.add( m_SearchTextField );
		simpleSearchPanel.add( m_SearchButton );
		simpleSearchPanel.add( m_ResetButton );
		//simpleSearchPanel.add( m_SearchCancelButton );
		
		JPanel shortestPathPanel = new JPanel( new GridBagLayout( ) );
		addPropertyField( shortestPathPanel, "", m_ToggleShowSPathCheckbox );
		addPropertyField( shortestPathPanel, "", m_ShowRoadsCheckbox );
		
		JPanel searchCategoriesPanel = new JPanel( new GridBagLayout( ) );
		addPropertyField( searchCategoriesPanel, "Category", m_SearchIconsComboBox );
		
		JPanel searchInContentsPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		searchInContentsPanel.add( m_SearchInNameCheckbox );
		searchInContentsPanel.add( m_SearchInDescCheckbox );
		searchInContentsPanel.add( m_SearchInURLCheckbox );
		
		JPanel searchInPanel	= new JPanel( new GridBagLayout( ) );
		addPropertyField( searchInPanel, "Search in", searchInContentsPanel );
		
		JPanel shortestPathFieldsPanel = new JPanel( new FlowLayout( ) );
		m_SPathX1TextField.setEnabled( false );
		m_SPathY1TextField.setEnabled( false );
		m_SPathX2TextField.setEnabled( false );
		m_SPathY2TextField.setEnabled( false );
		shortestPathFieldsPanel.add( new JLabel( "From (" ) );
		shortestPathFieldsPanel.add( m_SPathX1TextField );
		shortestPathFieldsPanel.add( new JLabel( ", " ) );
		shortestPathFieldsPanel.add( m_SPathY1TextField );
		shortestPathFieldsPanel.add( new JLabel( ") to (" ) );
		shortestPathFieldsPanel.add( m_SPathX2TextField );
		shortestPathFieldsPanel.add( new JLabel( ", " ) );
		shortestPathFieldsPanel.add( m_SPathY2TextField );
		shortestPathFieldsPanel.add( new JLabel( ")" ) );
		
		JPanel shortestPathInfoPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		shortestPathInfoPanel.add( m_SPathLengthLabel );
		
		addPropertyField( shortestPathPanel, "", shortestPathFieldsPanel );
		addPropertyField( shortestPathPanel, "", shortestPathInfoPanel );
		
		addPropertyField( base, "", simpleSearchPanel );
		addPropertyField( base, "", searchInPanel );
		addPropertyField( base, "", searchCategoriesPanel );
		addPropertyField( base, "", shortestPathPanel );
		
		searchPanel.setLayout( new BorderLayout( ) );
		searchPanel.add( BorderLayout.NORTH, base );
	}
	
	/**
	 * Arranges the login panel
	 * 
	 * @return the arranged panel
	 */
	private JPanel arrangeLoginPanel( )
	{
		JPanel loginPanel = new JPanel( new BorderLayout( ) );
		JPanel base     	= new JPanel( new GridBagLayout( ) );
		addPropertyField( base, "Login", m_LoginTextField );
		addPropertyField( base, "Password", m_PasswordTextField );
		
		JPanel buttonsPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		buttonsPanel.add( m_OKAuthButton );
		buttonsPanel.add( m_CancelAuthButton );
		addPropertyField( base, "", buttonsPanel );
		loginPanel.add( BorderLayout.NORTH, base );
		return loginPanel;
	}
	
	/**
	 * Arranges the info panel
	 * 
	 * @return the arranged panel
	 */
	private JPanel arrangeInfoPanel( )
	{
		JPanel infoPanel = new JPanel( new BorderLayout( ) );
		infoPanel.add( BorderLayout.CENTER, m_InfoCanvas );
		infoPanel.add( BorderLayout.SOUTH, m_InfoLabel );
		return infoPanel;		
	}
}
