package lib.applets;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.FontMetrics;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JApplet;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;


/**
 * @class lib.applets
 * 
 * Message box class. Displays modal dialogs with 
 * - mesasges with "OK" button only 
 * - confirmation messages with "Yes" and "No"
 * 
 * @author Tihomir Lazarov
 * @date   Jun 12, 2005
 */
public class MessageBox extends JDialog implements ActionListener 
{
	public static final int TYPE_CONFIRMATION	= 1;
	public static final int TYPE_INFO			= 2;
	public static final int TYPE_ERROR			= 3;
	
	private static final int RESULT_NONE		= 0;
	public static final int RESULT_OK			= 1;
	public static final int RESULT_YES			= 2;
	public static final int RESULT_NO			= 3;
	
	int m_iType 			= 0;
	String m_szMessage	= "";
	int m_iResult			= RESULT_NONE;;
	
	// GUI Controls
	JButton m_OKButton	= new JButton( "OK" );
	JButton m_YesButton	= new JButton( "Yes" );
	JButton m_NoButton	= new JButton( "No" );

	/**
	 * Constructs a modal dialog with a given message from a given type.
	 * 
	 * @param applet - the applet owner of the dialog
	 * @param iType - the type of the dialog (see TYPE_* constants)
	 * @param szMessage - the message to show
	 * 
	 * @throws java.awt.HeadlessException
	 */
	public MessageBox( JApplet applet, int iType, String szMessage ) 
		throws HeadlessException 
	{
		super( javax.swing.JOptionPane.getFrameForComponent( applet ), true );
		
		m_iType 			= iType;
		m_szMessage		= szMessage;
		
		arrangeControls( );
	}
	
	/**
	 * Returns the result depending on the button that's clicked on the
	 * the dialog
	 * 
	 * @return a result of one of these values: RESULT_OK, RESULT_YES, RESULT_NO
	 */
	public int getResult( )
	{
		return m_iResult;
	}
	
	/**
	 * This method is fired when a button is clicked on the dialog
	 * 
	 * @param event - the fired event
	 */
	public void actionPerformed( ActionEvent event )
	{
		if ( event.getSource( ).equals( m_OKButton ) ) {
			m_iResult = RESULT_OK;	
		} else if ( event.getSource( ).equals( m_YesButton ) ) {
			m_iResult = RESULT_YES;
		} else if ( event.getSource( ).equals( m_NoButton ) ) {
			m_iResult = RESULT_NO;
		}
		this.dispose( );
	}
	
	/* -- PRIVATE METHODS --------------------------------------------------- */
	
	/**
	 * Arranges the GUI controls depending on the type of the message box 
	 */
	private void arrangeControls( )
	{
		JLabel messageLabel = new JLabel( m_szMessage );
		FontMetrics fm = messageLabel.getFontMetrics( messageLabel.getFont( ) );
		
		int iTextWidth = fm.stringWidth( m_szMessage );
		iTextWidth 	  += ( int ) ( iTextWidth * 0.3 );
		iTextWidth		= iTextWidth > 800 ? 800 : iTextWidth;
		setSize( iTextWidth, 200 );
		getContentPane( ).setLayout( new BorderLayout( ) );
		
		JPanel messagePanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
		JPanel buttonsPanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
		String szTitle = "";
		
		m_OKButton.addActionListener( this );
		m_YesButton.addActionListener( this );
		m_NoButton.addActionListener( this );
		
		switch ( m_iType )
		{
			case TYPE_INFO:
				szTitle = "Information";
				buttonsPanel.add( m_OKButton );
				break;
			case TYPE_ERROR:
				szTitle = "Error";
				buttonsPanel.add( m_OKButton );
				break;
			case TYPE_CONFIRMATION:
				szTitle = "Confirmation";
				buttonsPanel.add( m_YesButton );
				buttonsPanel.add( m_NoButton );
				break;
		}
		 
		messagePanel.add( messageLabel );
		setTitle( szTitle );
		
		getContentPane( ).add( BorderLayout.CENTER, messagePanel );
		getContentPane( ).add( BorderLayout.SOUTH, buttonsPanel );

		this.setLocation( this.getOwner( ).getWidth( ) / 2 - getWidth( ) / 2, 
		                  this.getOwner( ).getHeight( ) / 2 - getHeight( ) / 2 );
		setVisible( true );
	}
}
