package lib.applets;

import lib.comm.AreaInfo;

/**
 * @class lib.applets
 * 
 * Stores the information that's needed for the search
 * view and is not represented as GUI controls.
 * 
 * @author Tihomir Lazarov
 * @date   Jun 11, 2005
 */
public class SearchView 
{
	AreaInfo m_Area1		= null;
	AreaInfo m_Area2		= null;
	
	/**
	 * Constructor. Resets the kept information.
	 */
	public SearchView( )
	{
		reset( );		
	}
	
	/**
	 * Resets the kept information
	 */
	public void reset( )
	{
		m_Area1	= null;
		m_Area2 = null;
	}
	
	/**
	 * Sets an area to the search view. If there are no areas
	 * set, the first area is initialized, otherwise - the second.
	 * If both areas are initialized and this method is invoked
	 * is called reset first and then the first area is initialized
	 * 
	 * @param areaInfo - the area to remember
	 */
	public void setArea( AreaInfo areaInfo )
	{
		if ( m_Area1 == null ) {
			// This is the first time of method invokation after a reset
			m_Area1 = areaInfo;
			return;
		}
		if ( m_Area2 == null ) {
			// The first area has been already initialized
			m_Area2 = areaInfo;
			return;
		}
		
		// Both areas have been initialized
		reset( );
		//	This can be done with a recursion, but we better not use any
		m_Area1 = areaInfo;  
	}
	
	/**
	 * Checks if both areas are initialized
	 * 
	 * @return true if both areas are initialized, false otherwise
	 */
	public boolean areAreasReady( )
	{
		return ( m_Area1 != null && m_Area2 != null );
	}
	
	/**
	 * Returns the two initialized areas. This method must be used only
	 * after verifying that areAreasReady returns true
	 * 
	 * @return an array of two AreaInfo objects
	 */
	public AreaInfo[] getAreas( )
	{
		AreaInfo[] areas = new AreaInfo[ 2 ];
		areas[ 0 ] = m_Area1;
		areas[ 1 ] = m_Area2;
		return areas;
	}
	
	/**
	 * Gets the number of areas that are already initialized
	 * @return the number of initialized areas
	 */
	public int getReadAreasCount( )
	{
		return ( m_Area1 == null ? 0 : 1 ) + ( m_Area2 == null ? 0 : 1 );
	}
}
