package lib.comm;

import java.io.Serializable;
import java.io.ObjectStreamException;
import java.util.Enumeration;
import java.util.Hashtable;


/**
 * @class lib.applets
 * 
 * The structure that holds the information given about the current image
 * 
 * @author Tihomir Lazarov
 * @date   Jun 5, 2005
 */
public class MapRequestInfo implements Serializable {
	// Serialized parameters
	public int ser_mapID				= 0;
	public int ser_prefAreaWidth	= 0;
	public int ser_prefAreaHeight	= 0;
	public int ser_viewerX			= 0;
	public int ser_viewerY			= 0;
	public int ser_viewerWidth		= 0;
	public int ser_viewerHeight	= 0;
	public int ser_mapWidth			= 0;
	public int ser_mapHeight		= 0;
	public int ser_zoomLevels		= 0;
	public int ser_zoomLevel		= 0;
	public Hashtable ser_areas		= null;
	public Hashtable ser_icons		= null;
	public Hashtable ser_groups	= null;
	public Hashtable ser_maps		= null;
	public boolean logged			= false;
	public long roadLength			= 0;
	
	/**
	 * Constructor.
	 */
	public MapRequestInfo( )
	{
		ser_areas 			= new Hashtable( );
		ser_icons			= new Hashtable( );
		ser_groups			= new Hashtable( );
		ser_maps				= new Hashtable( );
	}

	/**
	 * Adds an area to the list of areas (member value and the one for serialization)
	 * 
	 * @param areaInfo - the area info object
	 */	
	public void addArea( AreaInfo areaInfo ) 
	{
		ser_areas.put( areaInfo, "" );
	}
	
	/**
	 * Adds an icon to the list of current areas
	 * 
	 * @param iconInfo - the icon info object
	 */
	public void addIcon( IconInfo iconInfo )
	{
		ser_icons.put( iconInfo, "" );
	}
	
	/**
	 * Adds a gruop to the list of the groups
	 * 
	 * @param groupInfo - the GroupInfo object
	 */
	public void addGroup( GroupInfo groupInfo )
	{
		ser_groups.put( groupInfo, "" );
	}
	
	/**
	 * Adds a MapDefinition object to the list of current map
	 * 
	 * @param mapDefinition - the map definition object
	 */
	public void addMap( MapDefinition mapDefinition )
	{
		ser_maps.put( mapDefinition, "" );
	}

	/**
	 * Sets the viewer bounds
	 * 
	 * @param x - the viewer X relative to the current map zoom
	 * @param y - the viewer Y relative to the current map zoom
	 * @param width - the viewer width
	 * @param height - the viewer height
	 */	
	public void setViewerBounds( int x, int y, int width, int height )
	{
		ser_viewerX			= x;
		ser_viewerY 		= y;
		ser_viewerWidth	= width;
		ser_viewerHeight	= height;
	}
	
	/**
	 * Sets the map dimensions for the current map zoom level
	 * 
	 * @param width - the width of the map
	 * @param height - the height of the map
	 */
	public void setMapDimensions( int width, int height )
	{
		ser_mapWidth	= width;
		ser_mapHeight	= height;
	}
	
	/**
	 * Sets the number of zoom levels that are available 
	 * 
	 * @param zoomLevels - the number of zoom levels
	 */
	public void setZoomLevels( int zoomLevels )
	{
		ser_zoomLevels = zoomLevels;
	}
	
	/**
	 * Sets the current zoom level
	 * 
	 * @param zoomLevel - the value of the zoom level
	 */
	public void setZoomLevel( int zoomLevel )
	{
		ser_zoomLevel = zoomLevel;
	}

	/**
	 * Sets the current map ID
	 * 
	 * @param iMapID - the map ID to use
	 */
	public void setMapID( int iMapID ) 
	{
		ser_mapID = iMapID;
	}
	
	/**
	 * Sets the value of the preferred area width member variable
	 * 
	 * @param iWidth - the area width
	 */
	public void setPreferredAreaWidth( int iWidth )
	{
		ser_prefAreaWidth = iWidth;
	}
	
	/**
	 * Sets the value of the preferred area height member variable
	 * 
	 * @param iHeight - the area height
	 */
	public void setPreferredAreaHeight( int iHeight )
	{
		ser_prefAreaHeight = iHeight;
	}	
	
	/**
	 * Sets the current user state
	 * 
	 * @param isLogged - true or false
	 */
	public void setLogged( boolean isLogged )
	{
		logged = isLogged;
	}
	
	/**
	 * Sets the current road length
	 * 
	 * @param lRoadLength - the road length
	 */
	public void setRoadLength( long lRoadLength )
	{
		roadLength = lRoadLength;
	}
	
	/**
	 * Returns the value of the member value for viewer width
	 * 
	 * @return the viwer width
	 */
	public int getViewerWidth( ) 
	{
		return ser_viewerWidth;
	}
	
	/**
	 * Returns the value of the member value for viewer height
	 * 
	 * @return the viwer height
	 */
	public int getViewerHeight( )
	{
		return ser_viewerHeight;
	}
	
	/**
	 * Returns the viewer X position
	 * 
	 * @return - the X position
	 */
	public int getViewerX( )
	{
		return ser_viewerX;
	}
	
	/**
	 * Returns the viewer Y position
	 * 
	 * @return - the Y position
	 */
	public int getViewerY( )
	{
		return ser_viewerY;
	}
	
	/**
	 * Returns the number of zoom levels available for the current map
	 * 
	 * @return the number of zoom levels
	 */
	public int getZoomLevels( )
	{
		return ser_zoomLevels;
	}
	
	/**
	 * Returns the current zoom level for the map
	 * 
	 * @return the current zoom level value
	 */
	public int getZoomLevel( )
	{
		return ser_zoomLevel;
	}
	
	/**
	 * Returns the width of the current map zoom level
	 * 
	 * @return the width
	 */
	public int getMapWidth( )
	{
		return ser_mapWidth;
	}
	
	/**
	 * Returns the height of the current map zoom level
	 * 
	 * @return the height
	 */
	public int getMapHeight( )
	{
		return ser_mapHeight;
	}
	
	/**
	 * Returns the current areas as an enumeration of AreaInfo
	 * objects 
	 * 
	 * @return the areas
	 */
	public Enumeration getAreas( )
	{
		return ser_areas.keys( );
	}
	
	/**
	 * Returns the first area from the ser_areas
	 * hash. This method assumes that there's only
	 * one element in the hash. Otherwise returns null
	 * 
	 * @return AreaInfo object or null 
	 */
	public AreaInfo getFirstArea( )
	{
		if ( ser_areas.size( ) != 1 ) return null;
		Enumeration areas = getAreas( );
		if ( areas.hasMoreElements( ) ) {
			return ( AreaInfo ) areas.nextElement( );
		}
		return null;		
	}
	
	/**
	 * Indicates if the current user is logged or not
	 * 
	 * @return true if the user is logged, false otherwise
	 */
	public boolean isLogged( )
	{
		return logged;
	}
	
	/**
	 * Returns the current icons as an enumeration of IconInfo
	 * objects
	 * 
	 * @return the icons
	 */
	public Enumeration getIcons( )
	{
		return ser_icons.keys( );
	}
	
	/**
	 * Returns the current groups as an enumeration of GroupInfo objects
	 * 
	 * @return the groups
	 */
	public Enumeration getGroups( )
	{
		return ser_groups.keys( );
	}

	/**
	 * Returns the current maps as an enumeration of MapDefinition
	 * objects
	 * 
	 * @return the icons
	 */
	public Enumeration getMaps( )
	{
		return ser_maps.keys( );
	}
	
	/**
	 * Returns the ID of the current map
	 * 
	 * @return the ID of the map
	 */
	public int getMapID( )
	{
		return ser_mapID;
	}
	
	/**
	 * Returns the value of the preferred area width member variable
	 * 
	 * @return the area width
	 */
	public int getPreferredAreaWidth( )
	{
		return ser_prefAreaWidth;
	}
	
	/**
	 * Returns the value of the preferred area height member variable
	 * 
	 * @return the area height
	 */
	public int getPreferredAreaHeight( )
	{
		return ser_prefAreaHeight;
	}
	
	/**
	 * Returns the value of the road length member
	 * 
	 * @return the road length in meters
	 */
	public long getRoadLength( )
	{
		return roadLength;
	}
	
	/**
	 * Invoked when the object is deserialized
	 * 
	 * @return the instance of the object
	 * 
	 * @throws ObjectStreamException
	 */
	public Object readResolve( ) 
		throws ObjectStreamException
	{
		return this;
	}
	
	/**
	 * Dumps the information about the members of the class
	 */
	public String toString( )
	{
		String szDump = "";
		szDump += "*** Viewer dimensions (" + ser_viewerWidth + "x" + ser_viewerHeight + ")\n";
		szDump += "*** Pref area dimensions (" + ser_prefAreaWidth + "x" + ser_prefAreaHeight + ")\n";
		szDump += "*** Map ID: " + ser_mapID + "\n";
		szDump += "*** Areas count: " + ser_areas.size( ) + "\n";
		szDump += "*** Groups count: " + ser_groups.size( ) + "\n";
		szDump += "*** Icons count: " + ser_icons.size( ) + "\n";
		szDump += "*** Maps count: " + ser_maps.size( ) + "\n";
		return szDump;
	}
	
}
