package lib.db;

import java.sql.*;
import org.gjt.mm.mysql.Driver;
import lib.exceptions.DBManagerException;

/**
 * @class lib.db.Manager
 * 
 * MySQL database management
 * 
 * @author Tihomir Lazarov
 * @date   Sep 19, 2004
 */
public class DBManager {
	private static final String DRIVER = "jdbc:mysql:";
	private Statement m_Statement;
	private Connection m_Connection;
	
	/**
	 * Creates a Manager object
	 * 
	 * @param szHost		- MySQL host
	 * @param szDatabase	- database name
	 * @param szUserName	- database authorization user name
	 * @param szPassword - database authorization password
	 */
	public DBManager( String szHost, String szDatabase, String szUserName, String szPassword ) 
		throws DBManagerException 
	{
		try {
			Driver driver = new Driver();
			DriverManager.registerDriver( driver );
			m_Connection = DriverManager.getConnection( DRIVER + "//" + szHost + "/" + szDatabase + "?user=" + szUserName + "&password=" + szPassword );
			m_Statement  = m_Connection.createStatement();
		} catch ( SQLException e ) {
			throw new DBManagerException( "SQLException: " + e.getMessage( ), e );
		}
	}
	
	/**
	 * Executes a SELECT statement 
	 * 
	 * @param szQuery
	 * 
	 * @return a result set
	 */
	public ResultSet select( String szQuery ) throws DBManagerException
	{
		try {
			return m_Statement.executeQuery( szQuery );
		} catch ( SQLException e ) {
			throw new DBManagerException( "SQLException: " + e.getMessage( ), e );
		}
	}
	
	/**
	 * Executes an SQL statement
	 * 
	 * @param szQuery
	 * 
	 * @throws DBManagerException
	 */
	public void execute( String szQuery ) throws DBManagerException
	{
		try {
			m_Statement.execute( szQuery );
		} catch ( SQLException e ) {
			throw new DBManagerException( "SQLException: " + e.getMessage( ), e );
		}
	}	
	
	/**
	 * Returns a prepared statement
	 * 
	 * @param szStatement - the statement
	 * 
	 * @return the PrepareStatement object
	 */
	public PreparedStatement getPreparedStatement( String szStatement )
		throws DBManagerException
	{
		try {
			return m_Connection.prepareStatement( szStatement );
		} catch ( SQLException e ) {
			throw new DBManagerException( "SQLException: Failed getting the prepared statement: " + e.getMessage( ), e );
		}
	}
	
	/**
	 * Closes the connection
	 * 
	 * @throws DBManagerException
	 */
	public void closeConnection( ) throws DBManagerException
	{
		try {
			if ( m_Connection != null ) {
				m_Connection.close( );
			}
		} catch ( SQLException e ) {
			throw new DBManagerException( "SQLException: " + e.getMessage( ), e );
		}
	}	
}
