package lib.map.base;

/**
 * @author Tihomir Lazarov
 */
public class Rectangle {
	public int x;
	public int y;
	public int width;
	public int height;
	
	/**
	 * Creates an empty rectangle area
	 */
	public Rectangle( ) 
	{
		x 			= 0;
		y 			= 0;
		width 	= 0;
		height	= 0;
	}

	/** 
	 * Creates a rectangle by x,y, width, height
	 * 
	 * @param _x
	 * @param _y
	 * @param _width
	 * @param _height
	 */	
	public Rectangle( int _x, int _y, int _width, int _height ) 
	{
		x 			= _x;
		y 			= _y;
		width 	= _width;
		height 	= _height;
	}
	
	/**
	 * Creates a rectanglel by using other's rectangle dimensions
	 * 
	 * @param rect - the other rectangle
	 */
	public Rectangle( Rectangle rect ) 
	{
		x 			= rect.x;
		y			= rect.y;
		width		= rect.width;
		height	= rect.height;
	}
	
	/**
	 * Compares two rectangles by coordinates and dimensions.
	 * 
	 * @param r - the other rectangle
	 * 
	 * @return true if the coordinates and dimensions are the same. 
	 *         false otherwise.
	 */
	public boolean sameWith( Rectangle r ) 
	{
		return ( r.x == x && r.y == y && r.width == width && r.height == height );
	}
	
	/**
	 * Checks if a given point is inside the rectangle or on its border
	 * 
	 * @param _x - the X coordinate of the point
	 * @param _y - the Y coordinate of the point
	 * 
	 * @return true if the point is inside or on the border, false otherwise
	 */
	public boolean containsPoint( int _x, int _y )
	{
		return ( _x >= x && _x <= x + width &&
		         _y >= y && _y <= y + height );
	}
	
	/**
	 * Dumps the current rectangle coordinates and dimensions in a string
	 * 
	 * @return a string representation of the rectangle
	 */
	public String toString( ) 
	{
		return "(" + x + ", " + y + ", " + width + ", " + height + ")";
	}
}

