package lib.map.dynamic;

/**
 * @class lib.map.dynamic.Event
 * 
 * Describes an Event object with contains 2 strings:
 * o Event identifier
 * o Action 
 * 
 * The idea is to have a number of events for each Area object
 * which will trigger some actions.
 * 
 * @author Tihomir Lazarov
 * @date   Aug 16, 2004
 */
public class Event {
	private String m_szIdentifier;
	private String m_szAction;
	
	/**
	 * Constructs an 'Event' object
	 * 
	 * @param szIdentifier	- the identifier of the event
	 * @param szAction			- the event action
	 */
	public Event( String szIdentifier, String szAction ) {
		m_szIdentifier = szIdentifier;
		m_szAction 			= szAction;
	}
	
	/**
	 * Checks if the identifier of the current event is
	 * the same as a given identifier string
	 * 
	 * @param szIdentifier - the identifier to check
	 * 
	 * @return true if it matches the identifier, false otherwise
	 */
	public boolean hasIdentifier( String szIdentifier ) {
		return ( szIdentifier.equals( m_szIdentifier ) );
	}
	
	/**
	 * Returns the identifier of the event
	 * 
	 * @return the identifier
	 */
	public String getIdentifier( ) {
		return m_szIdentifier;
	}
	
	/**
	 * Returns the action of the event
	 * 
	 * @return the action
	 */
	public String getAction( ) {
		return m_szAction;
	}
}

