package lib.map.dynamic.cfg;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Enumeration;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import lib.exceptions.DBManagerException;
import lib.exceptions.DynamicMapConfigException;
import lib.map.dynamic.Area;
import lib.map.dynamic.Event;
import lib.map.base.Rectangle;
import lib.db.DBManager;
import lib.cfg.Constants;

/**
 * @author Tihomir Lazarov
 */
public class DynamicMapConfig {
	
	ArrayList m_Areas				= null;
	int       m_iMapID			= 0;
	DBManager m_DBManager		= null;
	
	/* -- Constants -------------------------------------------------------- */
	public static final int ATTRIBUTE_TYPE_EVENT 	= 1;
	public static final int ATTRIBUTE_TYPE_INFO  	= 2;
	public static final int ATTRIBUTE_TYPE_GEOMETRY	= 3;

	/**
	 * DynamicMapConfigHandler constructor
	 * 
	 * @param szFileName	- the XML definitions file name
	 */
	public DynamicMapConfig( DBManager manager, int iMapID ) 
	{
		m_Areas 		= new ArrayList( );
		m_DBManager	= manager;
		m_iMapID		= iMapID;
	}

	/**
	 * Returns the number of zoom levels
	 * 
	 * @return the number of zoom levels
	 */
	public ArrayList getAreas( ) 
		throws DynamicMapConfigException 
	{
		try {
			// Get the list of areas for this map ID
			ResultSet areaSet = m_DBManager.select( "SELECT * FROM areas WHERE MapID = " + m_iMapID );
			int iIndex = 0;
			while ( areaSet.next() ) {
				iIndex++;
			}
			
			int[] areaIDs = new int[ iIndex ];
			
			areaSet.beforeFirst();
			iIndex = 0;
			while ( areaSet.next() ) {
				int iAreaID = areaSet.getInt( "ID" );
				areaIDs[ iIndex++ ] = iAreaID;
			}
			
			for ( int i = 0; i < areaIDs.length; i++ ) {
				// Get the list of attributes for the current area
				int iAreaID	= areaIDs[ i ]; 
				ResultSet attributesSet = m_DBManager.select( "SELECT * FROM area_attributes WHERE AreaID = " + iAreaID );

				Hashtable attributes = new Hashtable( );				
				while ( attributesSet.next( ) ) {
					String szAttributeName 	= attributesSet.getString( "AttributeName" );
					String szAttributeValue	= attributesSet.getString( "AttributeValue" );
					
					attributes.put( szAttributeName, szAttributeValue );
				}
			
				int iAreaX			= Integer.parseInt( ( String )attributes.get( Constants.AREA_ATTRIBUTE_X 		) );
				int iAreaY			= Integer.parseInt( ( String )attributes.get( Constants.AREA_ATTRIBUTE_Y 		) );
				int iAreaWidth		= Integer.parseInt( ( String )attributes.get( Constants.AREA_ATTRIBUTE_WIDTH 	) );
				int iAreaHeight	= Integer.parseInt( ( String )attributes.get( Constants.AREA_ATTRIBUTE_HEIGHT ) );
				int iAreaZIndex	= Integer.parseInt( ( String )attributes.get( Constants.AREA_ATTRIBUTE_ZINDEX ) );
				int iAreaPolicy	= Integer.parseInt( ( String )attributes.get( Constants.AREA_ATTRIBUTE_POLICY ) );
					
				Area area = new Area( iAreaX, iAreaY, iAreaWidth, iAreaHeight, iAreaZIndex );
				area.setPolicy( iAreaPolicy );
				
				attributesSet.first( );
				
				while ( attributesSet.isAfterLast() == false ) {
					String szAttributeName 	= attributesSet.getString( "AttributeName" );
					String szAttributeValue	= attributesSet.getString( "AttributeValue" );
					int    iAttributeType	= attributesSet.getInt( "AttributeType" ); 
					
					if ( iAttributeType == ATTRIBUTE_TYPE_INFO ) {
						area.addInfo( szAttributeName, szAttributeValue );
					} else if ( iAttributeType == ATTRIBUTE_TYPE_EVENT ) {
						area.addEvent( new Event( szAttributeName, szAttributeValue ) );
					}
					
					attributesSet.next( );
				}
				
				/* Add the current definition in the list of zoom definitions */
				m_Areas.add( area );
			}
		} catch ( DBManagerException e ) {
			throw new DynamicMapConfigException( "DB manager exception: " + e.getMessage( ), e );
		} catch ( SQLException e ) {
			throw new DynamicMapConfigException( "SQL exception: " + e.getMessage( ), e );
		}
		
	   return m_Areas;
	}
	
	public void clearAreas( )
		throws DynamicMapConfigException 
	{
		try {
			// Get the areas for this map ID and delete the attributes for these areas
			System.out.println( "Clearing areas for map ID [" + m_iMapID + "]..." );
			ResultSet areas = m_DBManager.select( "SELECT * FROM areas WHERE MapID = " + m_iMapID );
			ArrayList queries = new ArrayList( );
			
			while ( areas.next( ) ) {
				int iAreaID = areas.getInt( "ID" );
				queries.add( "DELETE FROM area_attributes WHERE AreaID = " + iAreaID );
			}
			
			for ( int i = 0; i < queries.size( ); i++ ) {
				String szQuery = ( String )queries.get( i );
				m_DBManager.execute( szQuery );
			}
			
			// Delete the areas for this MapID
			m_DBManager.execute( "DELETE FROM areas WHERE MapID = " + m_iMapID );
		} catch ( DBManagerException e ) {
			throw new DynamicMapConfigException( "Failed getting areas to clear: " + e.getMessage( ), e );
		} catch ( SQLException e ) {
			throw new DynamicMapConfigException( "SQLException when clearning the areas for MapID [" + m_iMapID + "]: " + e.getMessage( ), e );
		}
	}
	
	/**
	 * Adds an area to a dynamic map
	 * 
	 * @param area - the area object that should be serialized
	 */
	public void addArea( Area area )
		throws DynamicMapConfigException 
	{
		Rectangle rect = area.getRectangle( );

		try {		
			m_DBManager.execute( "LOCK TABLES areas WRITE" );
			m_DBManager.execute( "INSERT INTO areas ( MapID ) VALUES ( " + m_iMapID + " )");
			ResultSet result = m_DBManager.select( "SELECT LAST_INSERT_ID( ) AS AreaID" );
			result.next( );
			int iAreaID = result.getInt( "AreaID" );
			m_DBManager.execute( "UNLOCK TABLES" );
			
			String szInsertBegin = "INSERT INTO area_attributes ( AreaID, AttributeName, AttributeValue, AttributeType ) VALUES( " + iAreaID; 
			m_DBManager.execute( szInsertBegin + ", '" + Constants.AREA_ATTRIBUTE_X 		+ "', " 	+ rect.x 				+ ", " + ATTRIBUTE_TYPE_GEOMETRY + " )" );
			m_DBManager.execute( szInsertBegin + ", '" + Constants.AREA_ATTRIBUTE_Y 		+ "', "	+ rect.y 				+ ", " + ATTRIBUTE_TYPE_GEOMETRY + " )" );
			m_DBManager.execute( szInsertBegin + ", '" + Constants.AREA_ATTRIBUTE_WIDTH 	+ "', " 	+ rect.width 			+ ", " + ATTRIBUTE_TYPE_GEOMETRY + " )" );
			m_DBManager.execute( szInsertBegin + ", '" + Constants.AREA_ATTRIBUTE_HEIGHT 	+ "', " 	+ rect.height 			+ ", " + ATTRIBUTE_TYPE_GEOMETRY + " )" );
			m_DBManager.execute( szInsertBegin + ", '" + Constants.AREA_ATTRIBUTE_ZINDEX 	+ "', " 	+ area.getZIndex( ) 	+ ", " + ATTRIBUTE_TYPE_GEOMETRY + " )" );
			m_DBManager.execute( szInsertBegin + ", '" + Constants.AREA_ATTRIBUTE_POLICY 	+ "', " 	+ area.getPolicy( )	+ ", " + ATTRIBUTE_TYPE_GEOMETRY + " )" );
			
			/* Print the info */
			Hashtable info = area.getInfo( );
			Enumeration keys = info.keys( );
			while ( keys.hasMoreElements( ) == true ) {
				String szKey 	= ( String )keys.nextElement( );
				String szValue	= ( String )info.get( szKey );
				
				PreparedStatement insertStatement = m_DBManager.getPreparedStatement( szInsertBegin + ", ?, ?, " + ATTRIBUTE_TYPE_INFO + " )" );
				insertStatement.setString( 1, szKey );
				insertStatement.setString( 2, szValue );
				insertStatement.execute( );
			}
			
			/* Print the events */
			ArrayList events = area.getEvents( );
			for ( int i = 0; i < events.size( ); i++ ) {
				Event event = ( Event ) events.get( i );   

				PreparedStatement insertStatement = m_DBManager.getPreparedStatement( szInsertBegin + ", ?, ?, " + ATTRIBUTE_TYPE_EVENT + " )" );
				insertStatement.setString( 1, event.getIdentifier( ) );
				insertStatement.setString( 2, event.getAction( ) );
				insertStatement.execute( );
			}
												 
		} catch ( DBManagerException e ) {
			throw new DynamicMapConfigException( e.getMessage( ), e );
		} catch ( SQLException e ) {
			throw new DynamicMapConfigException( e.getMessage( ), e );
		}
	}
	
}	
