package lib.map.ext;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import lib.comm.*;
import lib.db.DBManager;
import lib.exceptions.DBManagerException;

/**
 * @class lib.map.base
 * 
 * Gets maps from the database
 * 
 * @author Tihomir Lazarov
 * @date   Jun 1, 2005
 */
public class MapSelector {
	DBManager m_DBManager	= null;
	
	public MapSelector( DBManager db )
	{
		m_DBManager	= db;
	}
	
	/**
	 * Gets an array list of maps from the database
	 * 
	 * @return an array list of the available maps using the structure MapDefinition
	 */
	public ArrayList getMaps( )
	{
		return getMaps( 0 );
	}
	
	/**
	 * Gets an array list of maps from the database
	 * 
	 * @param iMapID - an optional map ID (0 for none)
	 * 
	 * @return an array list of the available maps using the structure MapDefinition
	 */
	public ArrayList getMaps( int iMapID )
	{
		try {
			String szQuery		= "SELECT * FROM maps" + ( iMapID > 0 ? " WHERE ID = " + iMapID : "" );
			ResultSet records = m_DBManager.select( szQuery );
			
			if ( records == null ) {
				return null;
			}
			
			ArrayList maps = new ArrayList( );
			
			while ( records.next( ) ) {
				MapDefinition map = new MapDefinition( );
				
				map.id 				= records.getInt( "ID" );
				map.name				= records.getString( "Name" );
				map.description	= records.getString( "Description" );
				map.scale			= records.getLong( "Scale" );
				map.isDefault		= records.getInt( "IsDefault" ) == 1 ? true : false;
				
				maps.add( map );
			}
			return maps;
		} catch ( DBManagerException e ) {
			return null;
		} catch ( SQLException e ) {
			return null;
		}
	}
}
