package lib.servlets;

import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.servlet.*;

import lib.cfg.Constants;
import lib.map.dynamic.DynamicMap;
import lib.exceptions.NullMapException;

/**
 * @class lib.servlets
 * 
 * Applies a search filter
 * 
 * @author Tihomir Lazarov
 * @date   Aug 22, 2004
 */
public class Search extends lib.servlets.ZoomMap {
	public void service( ServletRequest request, ServletResponse response )
			throws ServletException 
	{
		try {
			DynamicMap map = serviceBegin( request, response );
			
			// Search in... restrictions
			String szSearchInName	= request.getParameter( Constants.PARAM_SEARCH_IN_NAME );
			String szSearchInDesc	= request.getParameter( Constants.PARAM_SEARCH_IN_DESCRIPTION );
			String szSearchInURL		= request.getParameter( Constants.PARAM_SEARCH_IN_URL );
			ArrayList attrToSearchIn= new ArrayList( );
			if ( szSearchInName != null ) attrToSearchIn.add( Constants.AREA_ATTRIBUTE_NAME );
			if ( szSearchInDesc != null ) attrToSearchIn.add( Constants.AREA_ATTRIBUTE_DESCRIPTION );
			if ( szSearchInURL  != null ) attrToSearchIn.add( Constants.AREA_ATTRIBUTE_URL );
			
			// Search using the info in case there's any info passed as a query parameter
			String szQuery	= request.getParameter( Constants.PARAM_SEARCH_QUERY );
			if ( szQuery != null && szQuery.equals( "" ) == false ) {
				map.searchByInfo( szQuery, attrToSearchIn );	
			} else {
				// Reset the search by info
				map.resetSearchQuery( );
			}
			
			// Icon IDs
			String szIconIDs = request.getParameter( Constants.PARAM_SEARCH_ICON_IDS );
			if ( szIconIDs != null && szIconIDs.equals( "" ) == false ) {
				ArrayList iconIDs = new ArrayList( );
				// Convert the comma separated numbers to an array of list
				StringTokenizer token = new StringTokenizer( szIconIDs, "," );
				while ( token.hasMoreElements( ) ) {
					String szIconID = ( String ) token.nextElement( );
					int iIconID = Integer.parseInt( szIconID );
					iconIDs.add( new Integer( iIconID ) );
				}
				map.searchByIcons( iconIDs );
			} else {
				map.resetSearchByIcons( );
			}
			
			// Search for a shortest path if there are area IDs passed
			String szAreaID1 = request.getParameter( Constants.PARAM_SEARCH_AREA1 );
			String szAreaID2 = request.getParameter( Constants.PARAM_SEARCH_AREA2 );
			
			if ( szAreaID1 != null && szAreaID2 != null ) {
				long lAreaID1 = Long.parseLong( szAreaID1 );
				long lAreaID2 = Long.parseLong( szAreaID2 );
				map.searchShortestPath( lAreaID1, lAreaID2 );
			} else {
				// Otherwise reset the shortest path search
				map.resetShortestPathSearch( );
			}
			
			// Dump the whole graph (for debug only)
			String szShowGraph = request.getParameter( Constants.PARAM_SHOW_GRAPH );
			if ( szShowGraph != null ) {
				map.showGraph( true );
			} else {
				map.showGraph( false );
			}
			
			serviceEnd( request, response, map, map.getSnapshot( ) );
			
		} catch ( NullMapException e ) {
			throw new ServletException( e.getMessage( ) );
		}
	}
}
