package lib.servlets;

import javax.servlet.*;
import java.util.ArrayList;

import lib.map.dynamic.DynamicMap;
import lib.map.dynamic.Area;
import lib.map.base.Rectangle;
import lib.cfg.Constants;
import lib.servlets.AddArea;
import lib.exceptions.DynamicMapException;
import lib.exceptions.NullMapException;

/**
 * @class lib.servlets
 * 
 * Updates an area
 * 
 * @author Tihomir Lazarov
 * @date   Aug 22, 2004
 */
public class UpdateArea extends AddArea 
{
	public void service( ServletRequest request, ServletResponse response )
			throws ServletException 
	{
		try {
			DynamicMap map = serviceBegin( request, response );
			
			if ( isLogged( request ) ) {
				updateArea( request, map );
			} else {
				throw new ServletException( "This operation requires an authentication" );
			}
			serviceEnd( request, response, map, map.getSnapshot( ) );
			
		} catch ( NullMapException e ) {
			throw new ServletException( e.getMessage( ) );
		} catch ( DynamicMapException e ) {
			throw new ServletException( e.getMessage( ) );
		}
	}
	
	/**
	 * Update an area by given CGI parameters for a given map. The peculiarity
	 * here is that the width/height of the area are preserved as given by
	 * the user. 
	 * 
	 * @param request
	 * @param map
	 * 
	 * @throws ServletException
	 * @throws DynamicMapException
	 */
	private void updateArea( ServletRequest request, DynamicMap map ) 
				 throws ServletException, DynamicMapException 
	{
		String szUniqueID	= request.getParameter( Constants.AREA_PARAM_UNIQUE_ID );
		boolean bCentered	= false;
	
		if ( szUniqueID  == null ) {
		   System.out.println( "Update area: Insufficient arguments" );
			throw new ServletException( "Insufficient arguments to add an area" );
		}
		
		// Check if the updated area has the same X, Y coordinates
		String szAreaX   		= request.getParameter( Constants.AREA_ATTRIBUTE_X 				);
		String szAreaY   		= request.getParameter( Constants.AREA_ATTRIBUTE_Y 				);
		String szAreaWidth	= request.getParameter( Constants.AREA_ATTRIBUTE_WIDTH 			);
		String szAreaHeight	= request.getParameter( Constants.AREA_ATTRIBUTE_HEIGHT 			);
	
		if ( szAreaX     == null || szAreaY      == null || 
			  szAreaWidth == null || szAreaHeight == null ) {
			throw new ServletException( "Insufficient arguments to add an area" );
		}
	
		int iAreaX, iAreaY, iAreaWidth, iAreaHeight;
	
		try {
			iAreaX 			= Integer.parseInt( szAreaX );
			iAreaY 			= Integer.parseInt( szAreaY );
			iAreaWidth		= Integer.parseInt( szAreaWidth );
			iAreaHeight		= Integer.parseInt( szAreaHeight );
		} catch ( NumberFormatException e ) { 		
			throw new ServletException( "Invalid position arguments while adding an area: " + e.getMessage( ) );
		}
		
		long lUniqueID				= Long.parseLong( szUniqueID );
		ArrayList areas			= map.getAreas( );
		Rectangle viewer       	= map.getViewerRegion( );
		Rectangle realSizeArea	= map.scaleToRealSize( new Rectangle( iAreaX + viewer.x, 
																	                 iAreaY + viewer.y, 
																	                 iAreaWidth, 
																	                 iAreaHeight ) );
		for ( int i = 0; i < areas.size( ); i++ ) {
			Area area = ( Area ) areas.get( i );
			if ( area.getUniqueID() == lUniqueID ) {
				String szX = ( String ) area.getInfo( ).get( Constants.AREA_ATTRIBUTE_X );
				String szY = ( String ) area.getInfo( ).get( Constants.AREA_ATTRIBUTE_Y );
				if ( szX != null && szY != null && 
				     ( szX != szAreaX || szY != szAreaY ) ) {
					bCentered = true;
				}
			}
		}
		
		System.out.println( "Updating area" );
		map.removeAreaByUniqueID( lUniqueID );		
		super.addArea( request, map, bCentered );
	}	
}
