package lib.servlets;

import java.io.IOException;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.ServletException;
import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import java.awt.image.*;
import java.io.*;

import lib.db.DBManager;
import lib.cfg.Constants;
import lib.cfg.Config;
import lib.map.dynamic.DynamicMap;
import lib.map.dynamic.Snapshot;
import lib.map.dynamic.cfg.DynamicMapEnvironment;
import lib.map.dynamic.Icons;
import lib.user.Authentication;
import lib.encoder.ImageEncoder;
import lib.exceptions.DynamicMapException;
import lib.exceptions.NullMapException;
import lib.exceptions.DBManagerException;
import lib.exceptions.ConfigException;
import lib.exceptions.IconsException;

/**
 * @author Tihomir Lazarov
 */
public class ZoomMap implements Servlet {
	Snapshot m_LastSnapshot 		= null;
	DynamicMapEnvironment m_Env	= null;	
	int m_iViewerWidth				= 0;
	int m_iViewerHeight				= 0;				
	
	String m_szDBHost 				= "";
	String m_szDBName 				= "";
	String m_szDBUser 				= "";
	String m_szDBPass 				= "";	
	
	public void destroy() 
	{
	}

	public ServletConfig getServletConfig() 
	{
		return null;
	}

	public String getServletInfo() 
	{
		return null;
	}

	public void init(ServletConfig arg0) throws ServletException 
	{
		m_szDBHost = arg0.getInitParameter( Constants.PARAM_DB_HOST );
		m_szDBName = arg0.getInitParameter( Constants.PARAM_DB_NAME );
		m_szDBUser = arg0.getInitParameter( Constants.PARAM_DB_USER );
		m_szDBPass = arg0.getInitParameter( Constants.PARAM_DB_PASS );
	}

	public DynamicMap serviceBegin( ServletRequest request, ServletResponse response )
		throws ServletException 
	{
		HttpServletRequest req 	= ( HttpServletRequest )request;
		HttpSession session    	= req.getSession( true );
		DynamicMap map				= null;
		
		if ( request.getParameter( Constants.PARAM_RESET_MAP ) != null ) {
			System.out.println( "Resetting image..." );
			resetMap( session, map );
		}
		
		map = ( DynamicMap )session.getAttribute( Constants.SESSION_PARAM_MAP );
		
		System.out.println( "============== Service INIT START ===============" );
		
		if ( map == null ) {
			System.out.println( "ZoomMap: Cannot find the map inside the session. Creating a new one" );
			try {
				// Check if viewer dimensions are explicitly set
				int iViewerWidth			= 0;
				int iViewerHeight			= 0;
				String szViewerWidth		= request.getParameter( Constants.PARAM_VIEWER_WIDTH );
				String szViewerHeight	= request.getParameter( Constants.PARAM_VIEWER_HEIGHT );
				if ( szViewerWidth != null && szViewerHeight != null ) {
					iViewerWidth			= Integer.parseInt( szViewerWidth );
					iViewerHeight			= Integer.parseInt( szViewerHeight );
					System.out.println( "Viewer dimensions passed: " + iViewerWidth + "x" + iViewerHeight + "!" );
				} else {
					System.out.println( "Viewer dimensions NOT passed: " + szViewerWidth + "x" + szViewerHeight + "!" );
				}
				
				// Initialize a map
				DBManager dbmanager 	= new DBManager( m_szDBHost, m_szDBName, m_szDBUser, m_szDBPass );
				Config config    		= new Config( dbmanager );
				Icons icons				= new Icons( dbmanager, config );
				m_Env						= new DynamicMapEnvironment( dbmanager, config, icons, iViewerWidth, iViewerHeight );
				
				String szMapID			= request.getParameter( Constants.PARAM_MAP_ID );
				int iMapID				= szMapID == null ? -1 : Integer.parseInt( szMapID );
				map 						= new DynamicMap( m_Env, iMapID );
			} catch ( DynamicMapException e ) {
				throw new ServletException( "Dynamic Map Exception: " + e.getMessage( ) );
			} catch ( DBManagerException e ) {
				throw new ServletException( "DB Manager Exception: " + e.getMessage( ) );
			} catch ( ConfigException e ) {
				throw new ServletException( "Config exception: " + e.getMessage( ) );
			} catch ( IconsException e ) {
				throw new ServletException( "Icons exception: " + e.getMessage( ) );
			}
		} else {
			System.out.println( "ZoomMap: The map was found in the session" );
		}
		
		System.out.println( "============== Service INIT END ===============" );
		
		return map;
	}
	
	public void service( ServletRequest request, ServletResponse response ) 
		throws ServletException
	{
	}
		
	/**
	 * The end of the request - the current map and the current snapshot are stored
	 * into the session
	 * 
	 * @param request - request object
	 * @param response - respones object
	 * @param map - dynamic map object
	 * @param snapshot - snapshot object
	 * 
	 * @throws ServletException
	 */
	public void serviceEnd( ServletRequest request, ServletResponse response, DynamicMap map, Snapshot snapshot )
				throws ServletException 
	{
		HttpServletRequest req 	= ( HttpServletRequest )request;
		HttpSession session    	= req.getSession( true );
		
		try {
			// Store the current map state in the session
			session.setAttribute( Constants.SESSION_PARAM_MAP, map );
			
			// Check if there is any snapshot 
			if ( snapshot == null ) {
				snapshot	= ( Snapshot )session.getAttribute( Constants.SESSION_PARAM_SNAPSHOT );			
				if ( snapshot == null ) {
					snapshot = map.getSnapshot( );
				}
			}

			BufferedImage image 	= null;
			image                = snapshot.image;
			
			// Return the result image
			response.setContentType( Constants.DEFAULT_MAP_CONTENT_TYPE );
			OutputStream out 	 = response.getOutputStream( );
			ImageEncoder enc  = new ImageEncoder( );
			enc.JPEGEncode( image, out );			
			out.close( );
		} catch ( IOException e ) {
			throw new ServletException( e.getMessage( ) );
		} catch ( NullMapException e ) {
			throw new ServletException( e.getMessage( ) );
		}
		
		session.setAttribute( Constants.SESSION_PARAM_SNAPSHOT, snapshot );
	}
	
	/**
	 * Checks if the current request belongs to a logged user
	 * 
	 * @param request - the servlet request object
	 * 
	 * @return true if the user is logged, false otherwise
	 */
	protected boolean isLogged( ServletRequest request ) 
	{
		Authentication auth = new Authentication( null );
		return auth.isAuthenticated( request ); 
	}
	
	/**
	 * Resets the current session and the current map
	 * 
	 * @param session - the current session object
	 * @param map - the current DynamicMap object
	 */
	protected void resetMap( HttpSession session, DynamicMap map )
	{
		session.removeAttribute( Constants.SESSION_PARAM_AUTH 		);
		session.removeAttribute( Constants.SESSION_PARAM_SNAPSHOT 	);
		session.removeAttribute( Constants.SESSION_PARAM_MAP 			);
		map = null;		
	}
}



