package lib.utils;

import java.applet.Applet;

/**
 * @class lib.utils
 * 
 * ...Module description...
 * 
 * @author root
 * @date   May 27, 2005
 */
public class ProgressBar extends Progress {
	
	// The applet which status bar will be printed to
	private Applet m_Applet			= null;
	// The the message that will be printed before the percentage completeness
	private String m_szMessage		= "";
	private long m_lLastTime   	= 0;
	private int m_iLastProgress	= 0;
	private String m_szLastTime   = "";
	
	/**
	 * Progress bar constructor
	 * @param lAllSteps - number of all steps
	 * @param lStepSize - the size of a single step
	 * @param applet - the applet which status bar will be used
	 */
	public ProgressBar( Applet applet )
	{
		m_Applet = applet;
	}
	
	/**
	 * Initializes the progress bar
	 * 
	 * @param lAllSteps - the number of all steps
	 * @param lStepSize - the step size
	 * @param szMessage - the message that will be printed on the status
	 *                    bar before printing the percentage completeness
	 */
	public void init( long lAllSteps, long lStepSize, String szMessage )
	{
		super.init( lAllSteps, lStepSize );
		m_szMessage = szMessage;
	}
	
	/**
	 * Does a step forward in the progress bar
	 */
	public void step( ) 
	{
		super.step( );
		m_Applet.showStatus( m_szMessage + ": " + getProgress( ) + "% (ETC " + getETC( ) + ")" );
	}
	
	public void reset( )
	{
		m_lLastTime = 0;
		super.reset( );
	}
	
	/**
	 * Returns a string identifying the estimated time for completion
	 * of the task
	 * 
	 * @return a string with a human readable time format
	 */
	private String getETC( )
	{
		String szTime 		= "";	
		long lCurrentTime	= System.currentTimeMillis( );
		
		if ( m_lLastTime != 0 ) {
			if ( super.getProgress( ) != m_iLastProgress ) {
				long lDiff = lCurrentTime - m_lLastTime;
				lDiff *= ( 100 - super.getProgress( ) );
				
				lDiff /= 1000; // Ignore the milliseconds
				
				long lHours 	= lDiff / ( 60 * 60 ); lDiff -= lHours   * 60 * 60;
				long lMinutes	= lDiff / 60;          lDiff -= lMinutes * 60;
				long lSeconds  = lDiff;
				
				String szHours 	= lHours   < 10 ? "0" + lHours   : "" + lHours;
				String szMinutes	= lMinutes < 10 ? "0" + lMinutes : "" + lMinutes;
				String szSeconds	= lSeconds < 10 ? "0" + lSeconds : "" + lSeconds;
				
				szTime 				= szHours + ":" + szMinutes + ":" + szSeconds;
				m_szLastTime		= szTime;
				m_lLastTime 		= lCurrentTime;
				m_iLastProgress 	= super.getProgress( );
			}
		} else {
			m_lLastTime = lCurrentTime;
		}
		
		return m_szLastTime;
	}
}
