package test;

import java.util.ArrayList;
import java.util.Hashtable;

import lib.map.dynamic.AreaManager;
import lib.map.dynamic.Area;
import lib.map.dynamic.Event;
import lib.exceptions.AreaException;


/**
 * @class test
 * 
 * ...Module description...
 * 
 * @author Tihomir Lazarov
 * @date   Aug 16, 2004
 */
public class AreaManagerTest {
	
	static int m_iTestNumber = 0;
	
	public static void FAILED( String szMessage ) {
		System.err.println( "TEST #" + m_iTestNumber + " HAS FAILED: " + szMessage );
		System.exit( 1 );
	}
	
	public static void TEST( ) {
		m_iTestNumber++;
	}
	
	public static void DUMPAREAS( ArrayList areas ) {
		System.out.println( "------------ Areas -------------" );
		for ( int i = 0; i < areas.size( ); i++ ) {
			System.out.println( "***************" );
			System.out.println( ( ( Area )areas.get( i ) ).toString( ) );
			System.out.println( "" );
		}
	}
	
	public static void main( String[] args ) {
		AreaManager manager = new AreaManager( );

		Area area = new Area( 10, 10, 50, 50, 0 );
		area.addEvent( new Event( "onClick", "showURL" ) );
		area.addEvent( new Event( "onZoom", "zoom" ) );
		area.addInfo( "Description", "Some test area1" );
		area.addInfo( "Name", "Area1" );

		try {
			manager.addArea( area );
		} catch ( AreaException e ) {
			FAILED( "Exception caught: " + e.getMessage( ) );
		}

		/* --- Test ------*/
		TEST( );		
		ArrayList areas = manager.findAreasByPosition( 9, 9 );
		if ( areas != null ) {
			FAILED( "Some fake areas were found. Number of found areas: " + areas.size( ) );
		}
		
		area = new Area( 20, 20, 30, 30, 1 );
		area.addEvent( new Event( "onClick", "showURL" ) );
		area.addEvent( new Event( "onZoom", "zoom" ) );
		area.addInfo( "Description", "Some test area2" );
		area.addInfo( "Name", "Area2" );
		
		try {
			manager.addArea( area );
		} catch ( AreaException e ) {
			FAILED( "Exception caught by adding a second area: " + e.getMessage( ) );
		}
		
		area = new Area( 20, 20, 30, 30, 2 );
		area.addEvent( new Event( "onClick", "showURL" ) );
		area.addEvent( new Event( "onZoom", "zoom" ) );
		area.addInfo( "Description", "Some test area3" );
		area.addInfo( "Name", "..." );
		
		try {
			manager.addArea( area );
		} catch ( AreaException e ) {
			FAILED( "Exception caught by adding a second area: " + e.getMessage( ) );
		}
		
		
		/* --- Test ------*/
		TEST( );
		areas = manager.findAreasByPosition( 50, 50 );
		if ( areas == null ) {
			FAILED( "No areas were found on position" );
		}
		//DUMPAREAS( areas );
		
		/* --- Test ------*/
		TEST( );
		area = manager.findAreaByEventIdentifier( 40, 40, "onZoom" );
		if ( area == null || ( area != null && area.getZIndex( ) != 2 ) ) {
			FAILED( "No areas were found for event" );
		}
		
		//areas = new ArrayList( );
		//areas.add( area );
		//DUMPAREAS( areas );
		
		/* --- Test ------*/
		TEST( );
		Hashtable criteria = new Hashtable( );
		criteria.put( "Name", "Area1" );
		areas = manager.findAreasByInfo( criteria, true, true );
		
		if ( areas == null ) {
			FAILED( "No areas were found for criteria" );
		}
		//DUMPAREAS( areas );

		/* --- Test ------*/
		TEST( );
		criteria = new Hashtable( );
		criteria.put( "Name", "Area1" );
		areas = manager.findAreasByInfo( criteria, false, true );
		
		if ( areas == null ) {
			FAILED( "No areas were found for criteria" );
		}
		//DUMPAREAS( areas );

		/* --- Test ------*/
		TEST( );
		criteria = new Hashtable( );
		criteria.put( "Name", "aREa1" );
		areas = manager.findAreasByInfo( criteria, false, false );
		
		if ( areas == null ) {
			FAILED( "No areas were found for criteria" );
		}
		//DUMPAREAS( areas );
		
		/* --- Test ------*/
		TEST( );
		criteria = new Hashtable( );
		criteria.put( "Name", "aREa1" );
		areas = manager.findAreasByInfo( criteria, false, true );
		
		if ( areas != null ) {
			FAILED( "Areas were found for criteria but should not to" );
		}

		/* --- Test ------*/
		TEST( );
		criteria = new Hashtable( );
		criteria.put( "Name", "aREa" );
		criteria.put( "Description", "aREa" );
		areas = manager.findAreasByInfo( criteria, false, false );
		
		if ( areas == null ) {
			FAILED( "No areas were found for criteria" );
		}
		//DUMPAREAS( areas );

		/* --- Test ------*/
		TEST( );
		area = new Area( 20, 20, 30, 30, 0 );
		area.addEvent( new Event( "onClick", "showURL" ) );
		area.addEvent( new Event( "onZoom", "zoom" ) );
		area.addInfo( "Description", "Some test area3" );
		area.addInfo( "Name", "..." );
		
		boolean bCaughtExceptionForOverlapping = false;
		try {
			manager.addArea( area );
		} catch ( AreaException e ) {
			bCaughtExceptionForOverlapping = true;
			System.out.println( "Exception while adding an overlapping area: " + e.getMessage( ) );
		}
		if ( !bCaughtExceptionForOverlapping ) {
			FAILED( "Exception was NOT caught by adding an overlapping area" );
		}
		
	}
}
