package test;

import java.util.ArrayList;

import lib.road.RoadGraph;
import lib.road.RoadPoint;
import lib.road.ConnectionPoint;
import lib.road.ext.DBRoadGraph;

import lib.db.DBManager;
import lib.exceptions.DBManagerException;
import lib.exceptions.DBRoadGraphException;


/**
 * @class test
 * 
 * Tests graphs
 * 
 * @author Tihomir Lazarov
 * @date   May 12, 2005
 */
public class RoadGraphTest 
{
	public static final int MAP_ID		= 9999999;
	public static final String DB_HOST	= "localhost";
	public static final String DB_NAME	= "maps";
	public static final String DB_USER	= "root";
	public static final String DB_PASS	= "";
	
	public static void testGraphRoads( RoadGraph g ) 
	{
		ArrayList points = g.getPoints( );
		if ( g.hasDirectConnection( ( RoadPoint )points.get( 0 ), ( RoadPoint )points.get( 1 ) ) ) {
			System.out.println( "Yes, has a direct connection" );
		}
		
		ArrayList road = g.findShortestRoad( ( RoadPoint )points.get( 8 ), ( RoadPoint )points.get( 5 ) );
		if ( road != null ) {
			System.out.println( "Yes, has there is a road: " );
			for ( int i = 0; i < road.size(); i++ ) {
				RoadPoint p = ( RoadPoint ) road.get( i );
				System.out.println( "" + p.id + " " );
			}
		} else {
			System.out.println( "No, there's no road" );
		}
	}
	
	public static void saveToDB( RoadGraph g ) 
	{
		try {
			DBManager db 			= new DBManager( DB_HOST, DB_NAME, DB_USER, DB_PASS );
			DBRoadGraph dbGraph	= new DBRoadGraph( db );
			
			ArrayList graphs = new ArrayList( );
			graphs.add( g );
			System.out.println( "Saving to DB..." );
			dbGraph.save( MAP_ID, graphs );
			System.out.println( "Saved." );
		} catch ( DBManagerException e ) {
			System.out.println( "DBManagerException: " + e.getMessage( ) );
		} catch ( DBRoadGraphException e ) {
			System.out.println( "DBRoadGraphException: " + e.getMessage( ) );
		}
	}
	
	public static RoadGraph loadFromDB( )
	{
		try {
			DBManager db 			= new DBManager( DB_HOST, DB_NAME, DB_USER, DB_PASS );
			DBRoadGraph dbGraph	= new DBRoadGraph( db );
	
			System.out.println( "Loading from DB..." );
			ArrayList graphs = dbGraph.load( MAP_ID );
			System.out.println( "Loaded" );
			
			if ( graphs == null ) { return null; }
			
			RoadGraph graph = ( RoadGraph ) graphs.get( 0 );
			
			return graph;
		} catch ( DBManagerException e ) {
			System.out.println( "DBManagerException: " + e.getMessage( ) );
			return null;
		} catch ( DBRoadGraphException e ) {
			System.out.println( "DBRoadGraphException: " + e.getMessage( ) );
			return null;
		}
	}
	
	public static RoadGraph manualLoad( ) 
	{
		RoadGraph g = new RoadGraph( );
		int iPoints = 10;
		RoadPoint[] points = new RoadPoint[ iPoints ];
		
		for ( int i = 0; i < iPoints; i++ ) {
			RoadPoint p = new RoadPoint( i % 4 + i, i % 3 ); 
			g.addPoint( p );
			points[ i ] = p;
		}
		
		g.addConnection( points[0], points[1] );
		g.addConnection( points[1], points[2] );
		g.addConnection( points[1], points[3] );
		g.addConnection( points[2], points[3] );
		g.addConnection( points[2], points[4] );
		g.addConnection( points[3], points[4] );
		g.addConnection( points[4], points[5] );
		g.addConnection( points[5], points[6] );
		g.addConnection( points[6], points[7] );
		g.addConnection( points[6], points[3] );
		g.addConnection( points[4], points[8] );
		
		return g;
	}
	
	public static void cleanup( )
	{
		try {
			DBManager db 			= new DBManager( DB_HOST, DB_NAME, DB_USER, DB_PASS );
			DBRoadGraph dbGraph	= new DBRoadGraph( db );

			System.out.println( "Cleaning up..." );			
			dbGraph.removeForMap( MAP_ID );
			System.out.println( "Cleaned up" );
		} catch ( DBManagerException e ) {
			System.out.println( "DBManagerException: " + e.getMessage( ) );
		} catch ( DBRoadGraphException e ) {
			System.out.println( "DBRoadGraphException: " + e.getMessage( ) );
		}
	}
	
	public static void checkCorruption( RoadGraph graph )
	{
		ArrayList roadPoints = graph.getPoints( );
		
		System.out.println( "Checking for a corruption..." );
		
		for ( int i = 0; i < roadPoints.size( ); i++ ) {
			RoadPoint p1 = ( RoadPoint ) roadPoints.get( i );
			for ( int j = 0; j < p1.connections.size( ); j++ ) {
				ConnectionPoint conn = ( ConnectionPoint ) p1.connections.get( j );
				RoadPoint p2 = graph.getPointByID( conn.id );
		
				boolean bFound = false;
				for ( int k = 0; k < p2.connections.size( ); k++ ) {
					conn = ( ConnectionPoint ) p2.connections.get( k );
					if ( conn.id == p1.id ) {
						bFound = true;
						break;
					}
				}
				if ( !bFound ) {
					System.out.println( "Corrupted graph!!!!" );
					System.out.println( p1.toString( ) );
					System.out.println( p2.toString( ) );
					System.out.println( "========================" );
				}
			}
		}		
	}
	
	public static void main(String[] args) 
	{
		String szManualGraphOutput = "";
		String szDBGraphOutput     = ""; 
		
		RoadGraph g = manualLoad( );
		checkCorruption( g );		
		testGraphRoads( g );
		
		System.out.println( "-----------[ MANUAL ]-----------" );
		System.out.println( g.toString( ) );
		szManualGraphOutput = g.toString( ); 
		System.out.println( "-----------[ END MANUAL ]-----------" );
		
		/*
		g.removeConnection(g.getPointByID(1), g.getPointByID(2));
		g.removeConnection(g.getPointByID(1), g.getPointByID(3));
		g.removeConnection(g.getPointByID(1), g.getPointByID(0));
		g.removePoint(g.getPointByID(1));
		System.out.println( "-----------[ AFTER REMOVING ]-----------" );
		System.out.println( g.toString( ) );
		System.out.println( "-----------[ END MANUAL ]-----------" );
		/**/
		
		saveToDB( g );
		g = loadFromDB( );
		checkCorruption( g );
		System.out.println( "-----------[ FROM DB ]-----------" );
		System.out.println( g.toString( ) );
		szDBGraphOutput = g.toString( );
		System.out.println( "-----------[ FROM DB ]-----------" );
		testGraphRoads( g );
		
		cleanup( );
		
		if ( szManualGraphOutput.equals( szDBGraphOutput ) == true ) {
			System.out.println( "SUCCESS!!!" );
		} else {
			System.out.println( "FAILURE!" );
		}
		/**/
	}
}
