/*
 * Created on Aug 29, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package test;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

import lib.encoder.ImageEncoder;

/**
 * @class test
 * 
 * ...Module description...
 * 
 * @author root
 * @date   Aug 29, 2004
 */
public class RoadmapTest {
	
	BufferedImage m_Image = null;
	BufferedImage m_Copy  = null;
	Color m_Color = null;
	
	int BLACK = 0x000000;
	int WHITE = 0xffffff;
	
	public RoadmapTest( String szFileName, Color color )
	{
		System.out.println( "Loading image..." );
		MediaTracker tracker = new MediaTracker( new Container( ) );
		Image tmp = Toolkit.getDefaultToolkit( ).createImage( szFileName );
		tracker.addImage( tmp, 0 );
	
		try {	
			tracker.waitForAll( );
		} catch ( InterruptedException e ) {
			System.err.println( "InterruptedException: " + e.getMessage( ) );
			return;
		}
		
		System.out.println( "Reserving space for image..." );
		m_Image = new BufferedImage( tmp.getWidth( null ), tmp.getHeight( null ), BufferedImage.TYPE_INT_RGB );
		System.out.println( "Copying image..." );
		Graphics g = m_Image.getGraphics( );
		g.drawImage( tmp, 0, 0, null );
		
		System.out.println( "Highlighting..." );
		highlight( new Color( 170, 49, 21 ), new Color( 45, 20, 20 ) );
		
		g.setColor( Color.YELLOW );
		int start_x = 65;
		int start_y = 0;
		g.fillRect( start_x, start_y, 10, 10 );
		m_Image = walk( m_Image, start_x, start_y, 10 );
		
		System.out.println( "Saving..." );
		save( );
		
		System.out.println( "Done." );
	}
	
	public void separate( int hue_low, int hue_high, int saturation_low, int saturation_high, int value_low, int value_high ) {
		int iWidth  = m_Image.getWidth( );
		int iHeight = m_Image.getHeight( );
		for ( int i = 0; i < iWidth; i+=1 ) {
			for ( int j = 0; j < iHeight; j+=1 ) {
				Color c = new Color( m_Image.getRGB( i, j ) ); 
				HSV hsv = new HSV( c.getRed( ), c.getGreen(), c.getBlue( ) );
				if ( hsv.hue >= hue_low && hsv.hue <= hue_high &&
				     hsv.saturation >= saturation_low && hsv.saturation <= saturation_high && 
				     hsv.value >= value_low && hsv.value <= value_high ) {
					m_Image.setRGB( i, j, WHITE );
				} else {
					m_Image.setRGB( i, j, BLACK );
				}
			}
		}
	}
		
	
	public void highlight( Color sample, Color colorOffsets ) {
		separate( 0, 100, 50, 100, 50, 100 );
		removeGlitches( );
		//connectParts( 0, 0, 6, 7, 3, 3, false );
		//connectParts( 1, 1, 6, 7, 3, 3, false );
		//connectParts( 2, 2, 6, 7, 3, 3, false );
		
		//connectParts( 0, 0, 6, 8, 3, 3, true );
	}
	
	public Point reachNext( BufferedImage image, int x, int y, int radius ) {
		int best_x = x;
		int best_y = y;
		int best_radius = Integer.MAX_VALUE;
		int white = new Color( WHITE ).getRGB( );
			
		for ( int i = 1; i <= radius; i++ ) {
			for ( int deg = 0; deg < 360; deg++ ) {
				double rad = deg * Math.PI / 360.0f;
				int current_x = (int)(( x + i ) * Math.cos( rad ));
				int current_y = (int)(( x + i ) * Math.sin( rad ));
					
				if ( current_x >= 0 && current_x < image.getWidth( ) &&
				     current_y >= 0 && current_y < image.getHeight( ) &&
				     image.getRGB( current_x, current_y ) == white ) {
					best_radius = Math.min( best_radius, i );
					if ( best_radius == i ) {
						best_x = current_x;
						best_y = current_y;
					}
				}
			}
		}
		if ( best_radius == Integer.MAX_VALUE ) { 
			return null;
		}
		return new Point( best_x, best_y );
	}
	
	public BufferedImage walk( BufferedImage image, int x, int y, int radius ) {
		BufferedImage temp1 = new BufferedImage( image.getWidth( ), image.getHeight( ), BufferedImage.TYPE_INT_RGB );
		temp1.setData( image.getData( ) );
		boolean found = false;
		do {
			Point next = reachNext( temp1, x, y, radius );
			if ( next != null ) {
				Graphics g = temp1.getGraphics( );
				g.setColor( Color.RED );
				g.drawLine( x, y, next.x, next.y );
				temp1 = walk( temp1, next.x, next.y, radius );
			}
			found = next != null;
		} while ( found );
		return temp1;
	}
	
	public void connectParts( int offset_x, int offset_y, int threshold, int max_hops, int width, int height, boolean glitchesKiller ) {
		
		for ( int i = offset_x; i < m_Image.getWidth( ) - width; i+=width ) {
			for ( int j = offset_y; j < m_Image.getHeight( ) - height; j+=height ) {
				if ( energy( i, j, width, height, WHITE ) > threshold ) {
					Point up = find_energy( i, j, width, height, 0, -height, max_hops, WHITE );
					Point down = find_energy( i, j, width, height, 0, height, max_hops, WHITE );
					Point left = find_energy( i, j, width, height, -width, 0, max_hops, WHITE );
					Point right = find_energy( i, j, width, height, width, 0, max_hops, WHITE );
					
					Point topleft = find_energy( i, j, width, height, -width, -height, max_hops, WHITE ); 
					Point topright = find_energy( i, j, width, height, width, -height, max_hops, WHITE );
					Point bottomleft = find_energy( i, j, width, height, -width, height, max_hops, WHITE );
					Point bottomright = find_energy( i, j, width, height, width, height, max_hops, WHITE );

					Graphics g = m_Image.getGraphics( );
					g.setColor( new Color( WHITE ) );

					if ( up != null ) {
						g.fillRect( up.x, up.y, i - up.x + width, j - up.y + height );
					} else if ( down != null ) {
						g.fillRect( i, j, down.x - i + width, down.y - j + height );
					} else if ( left != null ) {
						g.fillRect( left.x, left.y, i - left.x + width, j - left.y + height );
					} else if ( right != null ) {
						g.fillRect( i, j, right.x - i + width, right.y - j + height );
					} else if ( topleft != null ) {
						g.drawLine( topleft.x, topleft.y, i, j );
					} else if ( topright != null ) {
						g.drawLine( topright.x, topright.y, i, j );
					} else if ( bottomleft != null ) {
						g.drawLine( bottomleft.x, bottomleft.y, i, j );
					} else if ( bottomright != null ) {
						g.drawLine( bottomright.x, bottomright.y, i, j );
					} else {
						if ( glitchesKiller ) {
							g.setColor( new Color( BLACK ) );
							g.fillRect( i, j, width, height );
						}
					}
				}
			}
		}
	}
	
	Point find_energy( int x, int y, int width, int height, int x_offset, int y_offset, int max_hops, int color ) {
		x += x_offset;
		y += y_offset;
		while ( x >= 0 && y >= 0 && x < m_Image.getWidth( ) - width && y < m_Image.getHeight( ) - height && max_hops > 0 ) {
		
			if ( energy( x, y, width, height, color ) > 6 ) {
				return new Point( x, y );
			}
			
			x += x_offset;
			y += y_offset;
			max_hops--;
		}
		
		return null;
	}
	
	public void removeGlitches( ) {
		for ( int i = 0; i < m_Image.getWidth( ) - 3; i+=3 ) {
			for ( int j = 0; j < m_Image.getHeight( ) - 3; j+=3 ) {
				if ( energy( i, j, 3, 3, WHITE ) > 6 ) {
					Graphics g = m_Image.getGraphics( );
					g.setColor( new Color( WHITE ) );
					g.fillRect( i, j, 3, 3 );
				}
			}
		}
		for ( int i = 0; i < m_Image.getWidth( ) - 3; i+=3 ) {
			for ( int j = 0; j < m_Image.getHeight( ) - 3; j+=3 ) {
				if ( energy( i, j, 3, 3, WHITE ) <= 3 ) {
					Graphics g = m_Image.getGraphics( );
					g.setColor( new Color( BLACK ) );
					g.fillRect( i, j, 3, 3 );
				}
			}
		}				
	}
	
	public int energy( int x, int y, int width, int height, int color ) {
		int iEnergy = 0;
		int rgb = new Color( color ).getRGB( );
		for ( int i = x; i <= x + width; i++ ) {
			for ( int j = y; j <= y + height; j++ ) {
				if ( m_Image.getRGB( i, j ) == rgb ) {
					iEnergy++;
				}
			}
		}
		return iEnergy;
	}
	
	public boolean aroundColor( Color color, Color sample, Color offsets ) {
		return ( aroundNumber( color.getRed( ), sample.getRed( ), offsets.getRed( ) ) &&
					aroundNumber( color.getGreen( ), sample.getGreen( ), offsets.getGreen( ) ) &&
					aroundNumber( color.getBlue( ), sample.getBlue( ), offsets.getBlue( ) ) );
	}
	
	public void highlightNot( Color color ) {
		int iWidth  = m_Image.getWidth( );
		int iHeight = m_Image.getHeight( );
		int iThresholdColor = color.getRGB( );
		for ( int i = 0; i < iWidth; i++ ) {
			for ( int j = 0; j < iHeight; j++ ) {
				int iCurrentColor = m_Image.getRGB( i, j );
				if ( iCurrentColor != iThresholdColor ) {
					m_Image.setRGB( i, j, iThresholdColor );
				}
			}
		}
	}
	
	///////////////////////////////////////////////////////////////////////
	public void sharpen( ) 
	{
		int[][] filter = {
			{  1,  1,  1 },
			{  0,  0,  0 },
			{ -1, -1, -1 },
		};
		int iFactor = 1;
		int iOffset = 127; 
		
		int iWidth  = m_Image.getWidth( );
		int iHeight = m_Image.getHeight( );
		for ( int i = 1; i < iWidth - 1; i += 1 ) {
			for ( int j = 1; j < iHeight - 1; j += 1 ) {
				applyFilter( m_Image, i, j, filter, iFactor, iOffset );
			}
		}
	}
	
	public void applyFilter( BufferedImage image, int x, int y, int filter[][], int iFactor, int iOffset ) 
	{
		int iReds 	= 0;
		int iGreens = 0;
		int iBlues  = 0;
		for ( int i = x - 1; i <= x + 1; i++ ) {
			int iMatrixX = i - x + 1;
			for ( int j = y - 1; j <= y + 1; j++ ) {
				Color c = new Color( image.getRGB( i, j ) );
				int iMatrixY = j - y + 1;
				int iCoef = filter[ iMatrixX ][ iMatrixY ];
				iReds += c.getRed( ) * iCoef;
				iGreens += c.getGreen( ) * iCoef;
				iBlues += c.getBlue() * iCoef;  
			}
		}
		if ( iFactor != 0 ) { 
			iReds /= iFactor; iReds += iOffset; 
			iGreens /= iFactor; iGreens += iOffset;
			iBlues /= iFactor; iBlues += iOffset;
		}
		iReds = Math.max( 0, Math.min( 255, iReds ) );
		iGreens = Math.max( 0, Math.min( 255, iGreens ) );
		iBlues = Math.max( 0, Math.min( 255, iBlues ) );
		image.setRGB( x, y, new Color( iReds, iGreens, iBlues ).getRGB( ) ); 
	}
	
	public boolean aroundNumber( int current, int sample, int buffer ) 
	{
		return ( sample >= current - buffer && sample <= current + buffer );
	}
	
	/////////////////////////////////////////////////////////////////////
	public void save( ) 
	{
		BufferedImage tmp = new BufferedImage( m_Image.getWidth( null ), m_Image.getHeight( null ), BufferedImage.TYPE_INT_RGB );
		Graphics g = tmp.getGraphics( );
		g.drawImage( m_Image, 0, 0, null );
		
		ImageEncoder enc = new ImageEncoder( );
		try {
			enc.JPEGEncode( tmp, new FileOutputStream( "./roadmap.jpg" ) );
		} catch ( IOException e ) {
			System.err.println( "Exception: " + e.getMessage( ) );
			return;
		}
	}

	/////////////////////////////////////////////////////////////////////
	public static void main(String[] args) {
		if ( args.length == 0 ) {
			Usage( );
			System.exit( 1 );
		}
		
		RoadmapTest test = new RoadmapTest( args[ 0 ], new Color( 255, 0, 0 ) );
	}
	
	//////////////////////////////////////////////////////////////////////
	public static void Usage( ) {
		System.out.println( "Usage: java RoadmapTest <FileName>" );
	}
	
	public class HSV {
		int hue	= 0;
		int saturation	= 0;
		int value	= 0;
		
		public HSV( int r, int g, int b ) {
			float red 	= r / 255.0f;
			float green	= g / 255.0f;
			float blue  = b / 255.0f;
			
			float x = Math.min( red, Math.min( green, blue ) );
			float v = Math.max( red, Math.max( green, blue ) );
			float f = 0;
			float i = 0;
			
			if ( v == x ) {
				hue = -1;
				saturation = 100 * 0;
				value = (int)(100 * x);
			}
			
			if ( red == x ) {
				f = green - blue;
			} else if ( green == x ) {
				f = blue - red;
			} else {
				f = red - green;
			}
			
			if ( red == x ) {
				i = 3;
			} else if ( green == x ) {
				i = 5;
			} else {
				i = 1;
			}
			
			hue = (int)(360 * ( i - f / ( v - x ) ));
			saturation = (int)(100 * ( v - x ) / v);
			value = (int)(100 * v); 
		}
	}
	
}
