package tools;

import java.io.*;
import java.util.Hashtable;

import lib.map.base.MappedImage;
import lib.db.DBManager;
import lib.exceptions.DBManagerException;

/**
 * Simple map builder
 * 
 * @author Tihomir Lazarov
 */
public class MapBuilder {
	
	String m_szInputFileName	= "";
	String m_szOutputDirectory = "";
	int    m_iMaxZoomLevel     = 0;
	DBManager m_Manager        = null;
	String m_szMapName			= "";
	String m_szMapDescription	= "";
	long   m_lScale				= 0;
	boolean m_bDefault         = false;
	
	/* --- MapBuilder class -------------------------------------------------*/
	public MapBuilder( DBManager manager,
	                   String szMapName,
	                   String szMapDescription,
	                   long   lScale,
	                   boolean bDefault,
	                   String szInputFileName, 
	                   String szOutputDirectory,
	                   int iMaxZoomLevel ) {
		m_Manager				= manager;
		m_szMapName				= szMapName;
		m_szMapDescription	= szMapDescription;
		m_lScale					= lScale;
		m_bDefault				= bDefault;
		m_szInputFileName 	= szInputFileName;
		m_szOutputDirectory	= szOutputDirectory;
		m_iMaxZoomLevel      = iMaxZoomLevel;
	}
	
	public static void _( String szMessage ) 
	{
		System.out.println( "*** " + szMessage );
	}
	
	public void build( ) 
			throws FileNotFoundException, 
			       InterruptedException, 
			       IOException,
					 IllegalArgumentException 
	{
				
		MappedImage.create( 	m_Manager,
									m_szMapName,
									m_szMapDescription,
									m_lScale,
									m_bDefault,
		                     m_szInputFileName, 
									m_szOutputDirectory,
									m_iMaxZoomLevel );
	}

	/* --- Main routine -----------------------------------------------------*/
	/**
	 * Main routine
	 * 
	 * @param args - command line arguments
	 */
	public static void main(String[] args) 
	{
		/* Check for enough arguments */
		if ( args.length != 11 ) {
			Usage( );
			System.exit( 1 );
		}
		
		/* Extract the parameters */
		Hashtable params = new Hashtable( );
		
		for ( int i = 0; i < args.length; i++) {
			if ( args[ i ].startsWith( "--" ) ) {
				int iEqSignIndex = args[ i ].indexOf( "=" );
				if ( iEqSignIndex == -1 ) {
					System.err.println( "WARNING: Inivalid parameter: " + args[ i ] );
				} else {
					String szName = args[ i ].substring( 2, iEqSignIndex );
					String szValue = args[ i ].substring( iEqSignIndex + 1 );
					params.put( szName, szValue );	
					//System.out.println( "[" + szName + "] = [" + szValue + "]" );				
				}
			}
		}
		
		try {
			DBManager manager  = new DBManager( 
											( String )params.get( "dbhost" ),
											( String )params.get( "dbname" ),
											( String )params.get( "dbuser" ),
											( String )params.get( "dbpassword" )
			                     );
			                     
			MapBuilder builder = new MapBuilder( manager,
															( String )params.get( "name" ),
															( String )params.get( "description" ),
															Long.parseLong( ( String )params.get( "scale" ) ),
															( ( String )params.get( "default" ) ).equals( "0" ) ? false : true,
															( String )params.get( "input" ),		// Input file 
															( String )params.get( "output" ),		// Output directory 
														    Integer.parseInt( ( String )params.get( "levels" ) )	// max zoom levels
														    );	
			builder.build( );
		} catch ( FileNotFoundException e ) {
			System.out.println( "File not found exception: " + e.getMessage( ) );
		} catch ( InterruptedException e ) {
			System.out.println( "Interrupted exception: " + e.getMessage( ) );
		} catch ( IOException e ) {
			System.out.println( "I/O exception: " + e.getMessage( ) );
		} catch ( IllegalArgumentException e ) {
			System.out.println( "Illegal argument exception: " + e.getMessage( ) );
		} catch ( DBManagerException e ) {
			System.out.println( "DB Manager exception: " + e.getMessage( ) );
		}
	}
	
	/**
	 * Prints application usage
	 */
	public static void Usage( ) 
	{
		System.out.println( "Usage: java MapBuilder\n" + 
		                                            "\t--input=<InputFile>\n" + 
		                                            "\t--output=<OutputDirectory>\n" + 
		                                            "\t--levels=<MaxZoomLevels>\n" + 
		                                            "\t--name=<MapName>\n" + 
		                                            "\t--description=<MapDescription>\n" +
		                                            "\t--scale=<distance in meters for 1px of the map>\n" +
		                                            "\t--default=<0 - not default, 1 - make it default map>\n" +  
		                                            "\t--dbhost=<DatabaseHost>\n" + 
		                                            "\t--dbuser=<DBUser>\n" + 
		                                            "\t--dbpassword=<DBPassword>\n" + 
		                                            "\t--dbname=<DBName>\n" );
	}
	
}
